/*
 * tiomap3430_pwr.c
 *
 * DSP-BIOS Bridge driver support functions for TI OMAP processors.
 *
 * Copyright (C) 2007-2008 Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 *  ======== _tiomap_pwr.c ========
 *  Description:
 *      Implementation of DSP wake/sleep routines.
 *
 *! Revision History
 *! ================
 *! 01-Nov-2007 HK: Added Off mode(Hibernation) support and DVFS support
 *! 05-Jan-2004 vp: Moved the file to platform specific folder and commented the
 *!		    code.
 *! 27-Mar-2003 vp: Added support for DSP boot idle mode.
 *! 06-Dec-2002 cring:  Added Palm support.
 *! 08-Oct-2002 rr:  Created.
 */

#include <mach-omap2/prcm-regs.h>
#include <mach-omap2/prm_regbits_34xx.h>
#include <mach-omap2/ti-compat.h>
/*  ----------------------------------- DSP/BIOS Bridge */
#include <dbdefs.h>
#include <errbase.h>
#include <cfg.h>
#include <drv.h>
#include <io_sm.h>
#include <chnl_sm.h>

/*  ----------------------------------- Trace & Debug */
#include <dbg.h>

/*  ----------------------------------- OS Adaptation Layer */
#include <mem.h>

/*  ----------------------------------- Platform Manager */
#include <brddefs.h>
#include <dev.h>
#include <iodefs.h>

/* ------------------------------------ Hardware Abstraction Layer */
#include <hw_defs.h>
#include <hw_dspssC64P.h>
#include <hw_prcm.h>
#include <hw_mmu.h>

#include <pwr_sh.h>

/*  ----------------------------------- specific to this file */
#include "_tiomap.h"
#include "_tiomap_pwr.h"
#include "_tiomap_util.h"

/*  ----------------------------------- Mini Driver */
#include "wmddeh.h"

#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
#ifndef CONFIG_OMAP3_PM
#include <mach/omap-pm.h>
#include <mach/board-3430sdp.h>
#else
#include <asm/arch/resource.h>
#endif
#endif
#endif
extern s32 dsp_test_sleepstate;

#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
#ifdef CONFIG_OMAP3_PM
extern struct constraint_handle *dsp_constraint_handle;
#endif
#endif
#endif
extern struct MAILBOX_CONTEXT mboxsetting;
extern unsigned short min_active_opp;

void DSPClkWakeupEventCtrl(u32 ClkId, bool enable)
{
	switch(ClkId) {
	case BPWR_GPTimer5:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_GPT5;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_GPT5;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_GPT5;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_GPT5;
		}
		break;
	case BPWR_GPTimer6:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_GPT6;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_GPT6;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_GPT6;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_GPT6;
		}
		break;
	case BPWR_GPTimer7:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_GPT7;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_GPT7;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_GPT7;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_GPT7;
		}
		break;
	case BPWR_GPTimer8:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_GPT8;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_GPT8;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_GPT8;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_GPT8;
		}
		break;
	case BPWR_MCBSP1:
		if(enable) {
			PM_IVA2GRPSEL1_CORE |= OMAP3430_GRPSEL_MCBSP1;
			PM_MPUGRPSEL1_CORE &= ~OMAP3430_GRPSEL_MCBSP1;
		} else {
			PM_MPUGRPSEL1_CORE |= OMAP3430_GRPSEL_MCBSP1;
			PM_IVA2GRPSEL1_CORE &= ~OMAP3430_GRPSEL_MCBSP1;
		}
		break;
	case BPWR_MCBSP2:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_MCBSP2;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP2;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_MCBSP2;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP2;
		}
		break;

	case BPWR_MCBSP3:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_MCBSP3;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP3;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_MCBSP3;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP3;
		}
		break;

	case BPWR_MCBSP4:
		if(enable) {
			PM_IVA2GRPSEL_PER |= OMAP3430_GRPSEL_MCBSP4;
			PM_MPUGRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP4;
		} else {
			PM_MPUGRPSEL_PER |= OMAP3430_GRPSEL_MCBSP4;
			PM_IVA2GRPSEL_PER &= ~OMAP3430_GRPSEL_MCBSP4;
		}
		break;
	case BPWR_MCBSP5:
		if(enable) {
			PM_IVA2GRPSEL1_CORE |= OMAP3430_GRPSEL_MCBSP5;
			PM_MPUGRPSEL1_CORE &= ~OMAP3430_GRPSEL_MCBSP5;
		} else {
			PM_MPUGRPSEL1_CORE |= OMAP3430_GRPSEL_MCBSP5;
			PM_IVA2GRPSEL1_CORE &= ~OMAP3430_GRPSEL_MCBSP5;
		}
		break;
	}

}


/*
 *  ======== handle_constraints_set ========
 *  	Sets new DSP constraint
 */
DSP_STATUS handle_constraints_set(struct WMD_DEV_CONTEXT *pDevContext,
				  IN void *pArgs)
{
#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
	u32 pConstraintVal;

	pConstraintVal = *(((u32 *)pArgs) + 1);
	/* Read the target value requested by DSP  */
	DBG_Trace(DBG_LEVEL7, "handle_constraints_set: opp requested = 0x%x\n",
						  pConstraintVal);

	/* Set the new constraint in resource framework */
#ifndef CONFIG_OMAP3_PM
	/*
	  * When Smartreflex is ON, DSP requires at least OPP level 3
	  * to operate reliably. So boost lower OPP levels to OPP3.
	  */
	if (pConstraintVal < min_active_opp) {
		pr_debug("DSPBRIDGE: VDD1 OPP%x elevated to OPP%x\n",
		pConstraintVal, min_active_opp);
		omap_pm_dsp_set_min_opp(min_active_opp);
	} else
		omap_pm_dsp_set_min_opp(pConstraintVal);
	return DSP_SOK;
#else
	if (pConstraintVal < min_active_opp) {
		pr_debug("DSPBRIDGE: VDD1 OPP%x elevated to OPP%x\n",
		pConstraintVal, min_active_opp);
		pConstraintVal = min_active_opp;
	}
	if (dsp_constraint_handle && constraint_set(dsp_constraint_handle,
			   pConstraintVal) == 0)
		return DSP_SOK;
	else {
		DBG_Trace(DBG_LEVEL7,
			 "handle_constraints_set: Constraint set failed\n");
		return DSP_EFAIL;
	}
#endif /*#ifndef CONFIG_OMAP3_PM*/
#endif /*#ifndef DISABLE_BRIDGE_DVFS*/
#endif /*#ifndef DISABLE_BRIDGE_PM*/
	return DSP_SOK;
}

/*
 *  ======== handle_hibernation_fromDSP ========
 *  	Handle Hibernation requested from DSP
 */
DSP_STATUS handle_hibernation_fromDSP(struct WMD_DEV_CONTEXT *pDevContext)
{
#ifndef DISABLE_BRIDGE_PM
	u16 usCount = 150;
	struct CFG_HOSTRES resources;
	DSP_STATUS status = DSP_SOK;
	enum HW_PwrState_t pwrState;
	u32 opplevel, prev_state;;
	struct IO_MGR *hIOMgr;
#ifdef BRIDGE_NTFY_PWRERR
	struct DEH_MGR *hDehMgr;
#endif /* #ifdef BRIDGE_NTFY_PWRERR */

	status = CFG_GetHostResources(
		 (struct CFG_DEVNODE *)DRV_GetFirstDevExtension(), &resources);
	if (DSP_FAILED(status))
		return status;

	prev_state = pDevContext->dwBrdState;
	pDevContext->dwBrdState = BRD_SLEEP_TRANSITION;
	HW_PWR_IVA2StateGet(resources.dwPrmBase, HW_PWR_DOMAIN_DSP,
			    &pwrState);
	/* Wait for DSP to move into Off state,  how much time should
	 * we wait? */
	while ((pwrState != HW_PWR_STATE_OFF) && --usCount) {
		msleep(10);
		HW_PWR_IVA2StateGet(resources.dwPrmBase, HW_PWR_DOMAIN_DSP,
				    &pwrState);
	}
	if (usCount == 0) {
		pDevContext->dwBrdState = prev_state;
		DBG_Trace(DBG_LEVEL7, "SleepDSP: Timed out Waiting for DSP"
			 " STANDBY %x \n", pwrState);
#ifdef BRIDGE_NTFY_PWRERR
		DEV_GetDehMgr(pDevContext->hDevObject, &hDehMgr);
		WMD_DEH_Notify(hDehMgr, DSP_PWRERROR, 0);
#endif /* #ifdef BRIDGE_NTFY_PWRERR */
		return WMD_E_TIMEOUT;
	}

	/* Save mailbox settings */
	status = HW_MBOX_saveSettings(resources.dwMboxBase);
	DBG_Trace(DBG_LEVEL6, "MailBoxSettings: SYSCONFIG = 0x%x\n",
		 mboxsetting.sysconfig);
	DBG_Trace(DBG_LEVEL6, "MailBoxSettings: IRQENABLE0 = 0x%x\n",
		 mboxsetting.irqEnable0);
	DBG_Trace(DBG_LEVEL6, "MailBoxSettings: IRQENABLE1 = 0x%x\n",
		 mboxsetting.irqEnable1);
	/* Turn off DSP Peripheral clocks and DSP Load monitor timer */
	status = DSP_PeripheralClocks_Disable(pDevContext, NULL);

	if (DSP_SUCCEEDED(status)) {
		/* Update the Bridger Driver state */
		pDevContext->dwBrdState = BRD_DSP_HIBERNATION;
#ifndef DISABLE_BRIDGE_DVFS
		status = DEV_GetIOMgr(pDevContext->hDevObject, &hIOMgr);
		if (DSP_FAILED(status))
			return status;
		IO_SHMsetting(hIOMgr, SHM_GETOPP, &opplevel);
		/* Set the OPP to low level before moving to OFF mode */
#ifndef CONFIG_OMAP3_PM
		if (opplevel != VDD1_OPP1) {
			DBG_Trace(DBG_LEVEL5,
				" DSP requested OPP = %d, MPU"
				" requesting low OPP %d instead\n",
				opplevel, VDD1_OPP1);
		}
		omap_pm_dsp_set_min_opp(VDD1_OPP1);
else
		if (opplevel != CO_VDD1_OPP1) {
			DBG_Trace(DBG_LEVEL5,
				" DSP requested OPP = %d, MPU"
				" requesting low OPP %d instead\n",
				opplevel, CO_VDD1_OPP1);
		}
		if (constraint_set(dsp_constraint_handle,
					  CO_VDD1_OPP1) != 0) {
			DBG_Trace(DBG_LEVEL7,
				"handle_hibernation_fromDSP:"
				"Constraint set failed\n");
			status = DSP_EFAIL;
		}
#endif
#endif
	} else {
		pDevContext->dwBrdState = prev_state;
		DBG_Trace(DBG_LEVEL7,
			 "handle_hibernation_fromDSP- FAILED\n");
	}
	return status;

#endif
	return DSP_SOK;

}
/*
 *  ======== SleepDSP ========
 *  	Put DSP in low power consuming state.
 */
DSP_STATUS SleepDSP(struct WMD_DEV_CONTEXT *pDevContext, IN u32 dwCmd,
		   IN void *pArgs)
{
	DSP_STATUS status = DSP_SOK;
#ifndef DISABLE_BRIDGE_PM
	struct CFG_HOSTRES resources;
#ifdef BRIDGE_NTFY_PWRERR
	struct DEH_MGR *hDehMgr;
#endif /* #ifdef BRIDGE_NTFY_PWRERR */
	u16 usCount = 150;
	u32 prev_state;
	enum HW_PwrState_t pwrState;
	enum HW_PwrState_t targetPwrState;

	status = CFG_GetHostResources(
		 (struct CFG_DEVNODE *)DRV_GetFirstDevExtension(), &resources);
	if (DSP_FAILED(status))
		goto func_end;
	DBG_Trace(DBG_LEVEL7, "SleepDSP- Enter function \n");

		/* next, check if sleep code is valid... */
	if ((dwCmd != PWR_DEEPSLEEP) && (dwCmd != PWR_EMERGENCYDEEPSLEEP)) {
		DBG_Trace(DBG_LEVEL7, "SleepDSP- Illegal sleep command\n");
		status = DSP_EINVALIDARG;
		goto func_end;
	}
	switch (pDevContext->dwBrdState) {
	case BRD_RUNNING:
		status = HW_MBOX_saveSettings(resources.dwMboxBase);
		if (dsp_test_sleepstate == HW_PWR_STATE_OFF) {
			IO_InterruptDSP2(pDevContext,
					 MBX_PM_DSPHIBERNATE);
			DBG_Trace(DBG_LEVEL7,
				 "SleepDSP - Sent hibernate "
				 "command to DSP\n");
			targetPwrState = HW_PWR_STATE_OFF;
		} else {
			IO_InterruptDSP2(pDevContext,
					 MBX_PM_DSPRETENTION);
			targetPwrState = HW_PWR_STATE_RET;
		}
		prev_state = BRD_RUNNING;
		pDevContext->dwBrdState = BRD_SLEEP_TRANSITION;
		break;
	case BRD_RETENTION:
		status = HW_MBOX_saveSettings(resources.dwMboxBase);
		if (dsp_test_sleepstate == HW_PWR_STATE_OFF) {
			IO_InterruptDSP2(pDevContext,
					 MBX_PM_DSPHIBERNATE);
			targetPwrState = HW_PWR_STATE_OFF;
		} else {
			status = DSP_SOK;
			goto func_end;
		}
		prev_state = BRD_RETENTION;
		pDevContext->dwBrdState = BRD_SLEEP_TRANSITION;
		break;
	case BRD_HIBERNATION:
	case BRD_DSP_HIBERNATION:
		/* Already in Hibernation, so just return */
		DBG_Trace(DBG_LEVEL7, "SleepDSP- DSP already in "
			 "hibernation\n");
		status = DSP_SOK;
		goto func_end;
	case BRD_STOPPED:
		DBG_Trace(DBG_LEVEL7,
			 "SleepDSP- Board in STOP state \n");
		status = DSP_SALREADYASLEEP;
		goto func_end;
	default:
		DBG_Trace(DBG_LEVEL7,
			 "SleepDSP- Bridge in Illegal state\n");
		status = DSP_EFAIL;
		goto func_end;
	}
	/* Get the PRCM DSP power domain status */
	HW_PWR_IVA2StateGet(resources.dwPrmBase, HW_PWR_DOMAIN_DSP,
			    &pwrState);
	/* Wait for DSP to move into Standby state,  how much time
	 * should we wait?*/
	while ((pwrState != targetPwrState) && --usCount) {
		msleep(10);
		HW_PWR_IVA2StateGet(resources.dwPrmBase, HW_PWR_DOMAIN_DSP,
				    &pwrState);
	}
	if (usCount == 0) {
		DBG_Trace(DBG_LEVEL7, "SleepDSP: Timed out Waiting for DSP"
			 " STANDBY %x \n", pwrState);
		status = WMD_E_TIMEOUT;
		pDevContext->dwBrdState = prev_state;
#ifdef BRIDGE_NTFY_PWRERR
		DEV_GetDehMgr(pDevContext->hDevObject, &hDehMgr);
		WMD_DEH_Notify(hDehMgr, DSP_PWRERROR, 0);
#endif /* #ifdef BRIDGE_NTFY_PWRERR */
		goto func_end;
	} else {
		DBG_Trace(DBG_LEVEL7, "SleepDSP: DSP STANDBY Pwr state %x \n",
			 pwrState);
		/* Update the Bridger Driver state */
		if (dsp_test_sleepstate == HW_PWR_STATE_OFF)
			pDevContext->dwBrdState = BRD_HIBERNATION;
		else
			pDevContext->dwBrdState = BRD_RETENTION;
		/* Turn off DSP Peripheral clocks  */
		status = DSP_PeripheralClocks_Disable(pDevContext, NULL);
		if (DSP_FAILED(status))
			DBG_Trace(DBG_LEVEL7, "SleepDSP- FAILED\n");
	}
func_end:
#endif
	return status;
}


/*
 *  ======== WakeDSP ========
 *  	Wake up DSP from sleep.
 */
DSP_STATUS WakeDSP(struct WMD_DEV_CONTEXT *pDevContext, IN void *pArgs)
{
	DSP_STATUS status = DSP_SOK;
#ifndef DISABLE_BRIDGE_PM
#ifdef DEBUG
	enum HW_PwrState_t pwrState;
	struct CFG_HOSTRES resources;

	status = CFG_GetHostResources(
		 (struct CFG_DEVNODE *)DRV_GetFirstDevExtension(), &resources);
	if (DSP_FAILED(status))
		return status;
#endif /* DEBUG */

	/* Check the BRD/WMD state, if it is not 'SLEEP' then return failure */
	if (pDevContext->dwBrdState == BRD_RUNNING ||
	    pDevContext->dwBrdState == BRD_STOPPED) {
		/* The Device is in 'RET' or 'OFF' state and WMD state is not
		 * 'SLEEP', this means state inconsistency, so return  */
		return DSP_SOK;
	}

	/* Send a wakeup message to DSP */
	CHNLSM_InterruptDSP2(pDevContext, MBX_PM_DSPWAKEUP);

#ifdef DEBUG
	HW_PWR_IVA2StateGet(resources.dwPrmBase, HW_PWR_DOMAIN_DSP,
			&pwrState);
	DBG_Trace(DBG_LEVEL7,
			"\nWakeDSP: Power State After sending Interrupt "
			"to DSP %x\n", pwrState);
#endif /* DEBUG */

	/* Set the device state to RUNNIG */
	pDevContext->dwBrdState = BRD_RUNNING;
#endif /* CONFIG_PM */
	return status;
}

/*
 *  ======== DSPPeripheralClkCtrl ========
 *  	Enable/Disable the DSP peripheral clocks as needed..
 */
DSP_STATUS DSPPeripheralClkCtrl(struct WMD_DEV_CONTEXT *pDevContext,
				IN void *pArgs)
{
	u32 extClk = 0;
	u32 extClkId = 0;
	u32 extClkCmd = 0;
	u32 clkIdIndex = MBX_PM_MAX_RESOURCES;
	u32 tmpIndex;
	u32 dspPerClksBefore;
	DSP_STATUS status = DSP_SOK;
	DSP_STATUS status1 = DSP_SOK;
	struct CFG_HOSTRES resources;
	u32 value;

	DBG_Trace(DBG_ENTER, "Entering DSPPeripheralClkCtrl \n");
	dspPerClksBefore = pDevContext->uDspPerClks;
	DBG_Trace(DBG_ENTER, "DSPPeripheralClkCtrl : uDspPerClks = 0x%x \n",
		  dspPerClksBefore);

	extClk = (u32)*((u32 *)pArgs);

	status = CFG_GetHostResources(
			(struct CFG_DEVNODE *)DRV_GetFirstDevExtension(),
			&resources);

	if (DSP_FAILED(status))
		return DSP_EFAIL;

	DBG_Trace(DBG_LEVEL3, "DSPPeripheralClkCtrl : extClk+Cmd = 0x%x \n",
		 extClk);

	extClkId = extClk & MBX_PM_CLK_IDMASK;

	/* process the power message -- TODO, keep it in a separate function */
	for (tmpIndex = 0; tmpIndex < MBX_PM_MAX_RESOURCES; tmpIndex++) {
		if (extClkId == BPWR_CLKID[tmpIndex]) {
			clkIdIndex = tmpIndex;
			break;
		}
	}
	/* TODO -- Assert may be a too hard restriction here.. May be we should
	 * just return with failure when the CLK ID does not match */
	/* DBC_Assert(clkIdIndex < MBX_PM_MAX_RESOURCES);*/
	if (clkIdIndex == MBX_PM_MAX_RESOURCES) {
		DBG_Trace(DBG_LEVEL7,
			 "DSPPeripheralClkCtrl : Could n't get clock Id for"
			 "clkid 0x%x \n", clkIdIndex);
		/* return with a more meaningfull error code */
		return DSP_EFAIL;
	}
	extClkCmd = (extClk >> MBX_PM_CLK_CMDSHIFT) & MBX_PM_CLK_CMDMASK;
	switch (extClkCmd) {
	case BPWR_DisableClock:
		/* Call BP to disable the needed clock */
		DBG_Trace(DBG_LEVEL3,
			 "DSPPeripheralClkCtrl : Disable CLK for \n");
		status1 = CLK_Disable(BPWR_Clks[clkIdIndex].intClk);
		status = CLK_Disable(BPWR_Clks[clkIdIndex].funClk);
		if (BPWR_CLKID[clkIdIndex] == BPWR_MCBSP1) {
			/* clear MCBSP1_CLKS, on McBSP1 OFF */
			value = __raw_readl(resources.dwSysCtrlBase + 0x274);
			value &= ~(1 << 2);
			__raw_writel(value, resources.dwSysCtrlBase + 0x274);
		} else if (BPWR_CLKID[clkIdIndex] == BPWR_MCBSP2) {
			/* clear MCBSP2_CLKS, on McBSP2 OFF */
			value = __raw_readl(resources.dwSysCtrlBase + 0x274);
			value &= ~(1 << 6);
			__raw_writel(value, resources.dwSysCtrlBase + 0x274);
		}
		DSPClkWakeupEventCtrl(BPWR_Clks[clkIdIndex].clkId, false);
		if ((DSP_SUCCEEDED(status)) && (DSP_SUCCEEDED(status1))) {
			(pDevContext->uDspPerClks) &=
				(~((u32) (1 << clkIdIndex)));
		} else {
			DBG_Trace(DBG_LEVEL7, "DSPPeripheralClkCtrl : Failed "
				 "to disable clk\n");
		}
		break;
	case BPWR_EnableClock:
		DBG_Trace(DBG_LEVEL3,
			 "DSPPeripheralClkCtrl : Enable CLK for \n");
		status1 = CLK_Enable(BPWR_Clks[clkIdIndex].intClk);
		status = CLK_Enable(BPWR_Clks[clkIdIndex].funClk);
		if (BPWR_CLKID[clkIdIndex] == BPWR_MCBSP1) {
			/* set MCBSP1_CLKS, on McBSP1 ON */
			value = __raw_readl(resources.dwSysCtrlBase + 0x274);
			value |= 1 << 2;
			__raw_writel(value, resources.dwSysCtrlBase + 0x274);
		} else if (BPWR_CLKID[clkIdIndex] == BPWR_MCBSP2) {
			/* set MCBSP2_CLKS, on McBSP2 ON */
			value = __raw_readl(resources.dwSysCtrlBase + 0x274);
			value |= 1 << 6;
			__raw_writel(value, resources.dwSysCtrlBase + 0x274);
		}
		DSPClkWakeupEventCtrl(BPWR_Clks[clkIdIndex].clkId, true);
		if ((DSP_SUCCEEDED(status)) && (DSP_SUCCEEDED(status1))) {
			(pDevContext->uDspPerClks) |= (1 << clkIdIndex);
		} else {
			DBG_Trace(DBG_LEVEL7,
				 "DSPPeripheralClkCtrl:Failed to Enable clk\n");
		}
		break;
	default:
		DBG_Trace(DBG_LEVEL3,
			 "DSPPeripheralClkCtrl : Unsupported CMD \n");
		/* unsupported cmd */
		/* TODO -- provide support for AUTOIDLE Enable/Disable
		 * commands */
	}
	return status;
}

/*
 *  ========PreScale_DSP========
 *  Sends prescale notification to DSP
 *
 */
DSP_STATUS PreScale_DSP(struct WMD_DEV_CONTEXT *pDevContext, IN void *pArgs)
{
#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
	u32 level;
	u32 voltage_domain;

	voltage_domain = *((u32 *)pArgs);
	level = *((u32 *)pArgs + 1);

	DBG_Trace(DBG_LEVEL7, "PreScale_DSP: voltage_domain = %x, level = "
		 "0x%x\n", voltage_domain, level);
	if  ((pDevContext->dwBrdState == BRD_HIBERNATION) ||
	    (pDevContext->dwBrdState == BRD_RETENTION) ||
	    (pDevContext->dwBrdState == BRD_DSP_HIBERNATION)) {
		DBG_Trace(DBG_LEVEL7, "PreScale_DSP: IVA in sleep. "
			 "No notification to DSP\n");
		return DSP_SOK;
	} else  if ((pDevContext->dwBrdState == BRD_RUNNING)) {
		/* Send a prenotificatio to DSP */
		DBG_Trace(DBG_LEVEL7,
			 "PreScale_DSP: Sent notification to DSP\n");
		IO_InterruptDSP2(pDevContext, MBX_PM_SETPOINT_PRENOTIFY);
		return DSP_SOK;
	} else {
		DBG_Trace(DBG_LEVEL7, "PreScale_DSP: Failed - DSP BRD"
			  " state in wrong state");
		return DSP_EFAIL;
	}
#endif /*#ifndef DISABLE_BRIDGE_DVFS*/
#endif  /*#ifndef DISABLE_BRIDGE_PM */
	return DSP_SOK;
}

/*
 *  ========PostScale_DSP========
 *  Sends postscale notification to DSP
 *
 */
DSP_STATUS PostScale_DSP(struct WMD_DEV_CONTEXT *pDevContext, IN void *pArgs)
{
#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
	u32 level;
	u32 voltage_domain;
	struct IO_MGR *hIOMgr;
	DSP_STATUS status = DSP_SOK;

	status = DEV_GetIOMgr(pDevContext->hDevObject, &hIOMgr);

	voltage_domain = *((u32 *)pArgs);
	level = *((u32 *)pArgs + 1);
	DBG_Trace(DBG_LEVEL7,
		 "PostScale_DSP: voltage_domain = %x, level = 0x%x\n",
		 voltage_domain, level);
	if  ((pDevContext->dwBrdState == BRD_HIBERNATION) ||
		    (pDevContext->dwBrdState == BRD_RETENTION) ||
		    (pDevContext->dwBrdState == BRD_DSP_HIBERNATION)  ||
		    (pDevContext->dwBrdState == BRD_SLEEP_TRANSITION)) {
		/* Update the OPP value in shared memory */
		IO_SHMsetting(hIOMgr, SHM_CURROPP, &level);
		DBG_Trace(DBG_LEVEL7,
			 "PostScale_DSP: IVA in sleep. Wrote to shared "
			 "memory \n");
		return DSP_SOK;
	} else  if ((pDevContext->dwBrdState == BRD_RUNNING)) {
		/* Update the OPP value in shared memory */
		IO_SHMsetting(hIOMgr, SHM_CURROPP, &level);
		/* Send a post notification to DSP */
		IO_InterruptDSP2(pDevContext, MBX_PM_SETPOINT_POSTNOTIFY);
		DBG_Trace(DBG_LEVEL7,
			 "PostScale_DSP: Wrote to shared memory Sent post"
			 " notification to DSP\n");
		return DSP_SOK;
	} else {
		DBG_Trace(DBG_LEVEL7, "PostScale_DSP: Failed - DSP BRD state "
			  "in wrong state");
		return DSP_EFAIL;
	}
#endif /*#ifndef DISABLE_BRIDGE_DVFS*/
#endif /*#ifndef DISABLE_BRIDGE_PM*/
	return DSP_SOK;
}

/*
 *  ========DSP_PeripheralClocks_Disable========
 *  Disables all the peripheral clocks that were requested by DSP
 */
DSP_STATUS DSP_PeripheralClocks_Disable(struct WMD_DEV_CONTEXT *pDevContext,
					IN void *pArgs)
{

	u32 clkIdx;
	DSP_STATUS status = DSP_SOK;
	struct CFG_HOSTRES resources;
	u32 value;

	status = CFG_GetHostResources(
			(struct CFG_DEVNODE *)DRV_GetFirstDevExtension(),
			&resources);

	for (clkIdx = 0; clkIdx < MBX_PM_MAX_RESOURCES; clkIdx++) {
		if (((pDevContext->uDspPerClks) >> clkIdx) & 0x01) {
			/* Disables the interface clock of the peripheral */
			status = CLK_Disable(BPWR_Clks[clkIdx].intClk);
			if (BPWR_CLKID[clkIdx] == BPWR_MCBSP1) {
				/* clear MCBSP1_CLKS, on McBSP1 OFF */
				value = __raw_readl(resources.dwSysCtrlBase
								+ 0x274);
				value &= ~(1 << 2);
				__raw_writel(value, resources.dwSysCtrlBase
								+ 0x274);
			} else if (BPWR_CLKID[clkIdx] == BPWR_MCBSP2) {
				/* clear MCBSP2_CLKS, on McBSP2 OFF */
				value = __raw_readl(resources.dwSysCtrlBase
								+ 0x274);
				value &= ~(1 << 6);
				__raw_writel(value, resources.dwSysCtrlBase
								+ 0x274);
			}
			if (DSP_FAILED(status)) {
				DBG_Trace(DBG_LEVEL7,
					 "Failed to Enable the DSP Peripheral"
					 "Clk 0x%x \n", BPWR_Clks[clkIdx]);
			}
			/* Disables the functional clock of the periphearl */
			status = CLK_Disable(BPWR_Clks[clkIdx].funClk);
			if (DSP_FAILED(status)) {
				DBG_Trace(DBG_LEVEL7,
					 "Failed to Enable the DSP Peripheral"
					 "Clk 0x%x \n", BPWR_Clks[clkIdx]);
			}
		}
	}
	return status;
}

/*
 *  ========DSP_PeripheralClocks_Enable========
 *  Enables all the peripheral clocks that were requested by DSP
 */
DSP_STATUS DSP_PeripheralClocks_Enable(struct WMD_DEV_CONTEXT *pDevContext,
				      IN void *pArgs)
{
	u32 clkIdx;
	DSP_STATUS status = DSP_SOK;
	struct CFG_HOSTRES resources;
	u32 value;

	status = CFG_GetHostResources(
			(struct CFG_DEVNODE *)DRV_GetFirstDevExtension(),
			&resources);

	for (clkIdx = 0; clkIdx < MBX_PM_MAX_RESOURCES; clkIdx++) {
		if (((pDevContext->uDspPerClks) >> clkIdx) & 0x01) {
			/* Enable the interface clock of the peripheral */
			status = CLK_Enable(BPWR_Clks[clkIdx].intClk);
			if (BPWR_CLKID[clkIdx] == BPWR_MCBSP1) {
				/* set MCBSP1_CLKS, on McBSP1 ON */
				value = __raw_readl(resources.dwSysCtrlBase
								+ 0x274);
				value |= 1 << 2;
				__raw_writel(value, resources.dwSysCtrlBase
								+ 0x274);
			} else if (BPWR_CLKID[clkIdx] == BPWR_MCBSP2) {
				/* set MCBSP2_CLKS, on McBSP2 ON */
				value = __raw_readl(resources.dwSysCtrlBase
								+ 0x274);
				value |= 1 << 6;
				__raw_writel(value, resources.dwSysCtrlBase
								+ 0x274);
			}
			if (DSP_FAILED(status)) {
				DBG_Trace(DBG_LEVEL7,
					 "Failed to Enable the DSP Peripheral"
					 "Clk 0x%x \n", BPWR_Clks[clkIdx]);
			}
			/* Enable the functional clock of the periphearl */
			status = CLK_Enable(BPWR_Clks[clkIdx].funClk);
			if (DSP_FAILED(status)) {
				DBG_Trace(DBG_LEVEL7,
					 "Failed to Enable the DSP Peripheral"
					 "Clk 0x%x \n", BPWR_Clks[clkIdx]);
			}
		}
	}
	return status;
}

/*
 *  ======== tiomap3430_bump_dsp_opp_level ========
 *  	This function bumps DSP OPP level if it is OPP1
 */
DSP_STATUS tiomap3430_bump_dsp_opp_level(void)
{
#ifndef DISABLE_BRIDGE_PM
#ifndef DISABLE_BRIDGE_DVFS
	struct WMD_DEV_CONTEXT *dwContext;
	struct DEV_OBJECT *hDevObject =
				(struct DEV_OBJECT *)DRV_GetFirstDevObject();

	if (DSP_FAILED(DEV_GetWMDContext(hDevObject,
			   (struct WMD_DEV_CONTEXT **)&dwContext)))
				return DSP_EFAIL;

	if (dwContext->dwBrdState == BRD_DSP_HIBERNATION ||
				dwContext->dwBrdState == BRD_HIBERNATION) {
#ifndef CONFIG_OMAP3_PM
		/* Increase OPP level before waking up the DSP */
		omap_pm_dsp_set_min_opp(min_active_opp);
#else
		if (!dsp_constraint_handle)
			return DSP_EHANDLE;
		/* Increase OPP level before waking up the DSP */
		if (constraint_set(dsp_constraint_handle,
			   min_active_opp) != 0) {
			DBG_Trace(DBG_LEVEL7, "CHNLSM_InterruptDSP: "
					 "Constraint set failed\n");
			return DSP_EFAIL;
		}
#endif
	}
#endif
#endif
	return DSP_SOK;
}

