/*  ============================================================================
*   File    DmmCopy.c
*
*   Path    $(TESTSUITEROOT)i/Analysis/dmmcopy/
*
*   Desc  wdmmcopy demonstrates usage of the Dynamic Memory Mapping APIs
*      by creating two MPU buffers, mapping them to the DSP virtual
*      address space, and copying data on the DSP from one buffer to
*      another.
*
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2004
*
*   ============================================================================
*   Revision History
*
*   Nov 24, 2004  REF= Based on the DMMCOPY sample             Ramesh Gupta G
*      Original Version.
*   ============================================================================
*/

#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>
#include <sys/time.h>

/* Application sepcific attributes */
#define DEFAULTBUFSIZE      0x19000    /* Default buffer size in bytes. */
#define DEFAULTDSPUNIT      0       /* Default unit number for the board. */
#define DEFAULTNBUFS        1       /* Default # of buffers. */
#define DMMCOPY_TASK_ID TEXT("dmmcopy")
#define ARGSIZE             32
#define DSPWORDSIZE         1       /* 1 DSP word = 2 bytes. */
#define DEFAULTDSPEXEC	"msg_nortdx_ext_2430.dof64P"

UINT    g_dwDSPWordSize=1; // default for 2430
/* Messaging DSP commands */
#define DMM_SETUPBUFFERS    0xABCD
#define DMM_WRITEREADY      0xADDD


/* dmmcopy task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    DSP_HNODE           hNode;          /* Handle to node. */
    INT 	     	numIterations;
    INT	     		AccessFlag;
} DMMCOPY_TASK;



/* Forward declarations: */

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeDMMProcessor(DMMCOPY_TASK *copyTask,char *dspImage);
static DSP_STATUS InitializeDMMNode(DMMCOPY_TASK *copyTask);
static DSP_STATUS CleanupDMMProcessor(DMMCOPY_TASK *copyTask);
static DSP_STATUS CleanupDMMNode(DMMCOPY_TASK *copyTask);
static DSP_STATUS RunDMMTask(DMMCOPY_TASK *copyTask,ULONG size);
    

/*
 *  ======== main ========
 */
INT DSPDMMCOPY_main(INT argc, char ** argv)
{
    DMMCOPY_TASK    dmmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    char 	     *dspImage;
    char 	     *buff_size;
    ULONG 	BufferSize=0;
    INT 	       opt = 0;
    INT 	     	inumIterations;
    INT	     	iAccessFlag;

    DspManager_Open(argc, NULL);

    /* Process command line arguments, open data files: */
	  
	  optind = 0;
          while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:s:a:b:n:ZPR:")) != EOF)
           {
                switch (opt)
               {
                        case 'v':
                          /* verbose */
                        TST_setVerbose(TRUE);
                        break;

                        case 'l':
            /* use log and store with name location */
                        strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
                        TSTLOG_appendLog(pHandle, "Log File Set to: %s\n", szLogName);
                        break;

                        case 'x':
            
	  /* use given executable */
                        dspImage = optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "Executable :%s\n", optarg);
                        break;

			case 'a':
         	  /* Access flag */
			iAccessFlag = CSL_Atoi(optarg);
			break;
		
	         /* Buffer size is */
                        case 'b':
                        buff_size=optarg;
                        TSTLOG_appendLog(pHandle, "Buffer Size is set to :%s\n", optarg);
                        break;

              /* Number of Iterations */          
			case 'n':
                        inumIterations=CSL_Atoi(optarg);
                        TSTLOG_appendLog(pHandle, "Number of iterations is set to :%s\n", optarg);
                        break;


                        default:
                        TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
                        break;

                }

          }
		dmmcopyTask.hProcessor = NULL;
    	dmmcopyTask.hNode = NULL;
    	dmmcopyTask.numIterations = inumIterations;
    	dmmcopyTask.AccessFlag = iAccessFlag;

        BufferSize= atoi(buff_size);


    if (DSP_SUCCEEDED(status)) {
        /* Perform processor level initialization. */
        status = InitializeDMMProcessor(&dmmcopyTask,dspImage);
    }

    if (DSP_SUCCEEDED(status)) {

        /* Perform node level initialization. */
        status = InitializeDMMNode(&dmmcopyTask);

        if (DSP_SUCCEEDED(status)) {

            /* Run task. */
            status = RunDMMTask(&dmmcopyTask,BufferSize);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle,"RunDMMTask succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "RunDMMTask failed.\n");
            }
        }
    }

    status = CleanupDMMNode(&dmmcopyTask);
	if(DSP_SUCCEEDED(status))
	{
                TSTLOG_appendLog(pHandle,"CleanupDMMNode succeeded.\n");
	}
	else
	{
                TSTLOG_appendLog(pHandle,"CleanupDMMNode Failed.\n");
	}
    status = CleanupDMMProcessor(&dmmcopyTask);
	if(DSP_SUCCEEDED(status))
	{
                TSTLOG_appendLog(pHandle,"CleanupDMMProcessor succeeded.\n");
	}
	else
	{
                TSTLOG_appendLog(pHandle,"CleanupDMMProcessor Failed.\n");
	}

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeDMMProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeDMMProcessor(DMMCOPY_TASK *copyTask,char *dspImage)
{
    DSP_STATUS          status = DSP_EFAIL;
    INT			argc = 1;
    const CHAR *	argv = DEFAULTDSPEXEC;
    struct DSP_PROCESSORINFO	dspInfo;
    INT			numProcs;
    UINT		index=0;
    INT			procId;

    /* Attach to DSP */
	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index, &dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO), &numProcs)))
	{
		if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64)
		{
			if(dspInfo.uProcessorType == DSPTYPE_55)
            		{
		                g_dwDSPWordSize = 2;
            		}
            		else
            		{
                		g_dwDSPWordSize = 1;
            		}

			procId = index;
			status = DSP_SOK;	
			break;	
		}
		index++;	
	}

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&copyTask->hProcessor);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
	else
     TSTLOG_appendLog(pHandle,"Failed to get the desired processor \n");


    if (DSP_SUCCEEDED(status))
    {
            TSTLOG_appendLog(pHandle, " DSP Image: %s.\n", dspImage) ;

            status = DSPProcessor_Load(copyTask->hProcessor,argc,&dspImage,NULL) ;

            if (!DSP_SUCCEEDED(status))
            {
                    TSTLOG_appendLog(pHandle, "DSPProcessor_Load failed. and status is %x\n",status) ;
            }

            status = DSPProcessor_Start(copyTask->hProcessor) ;

            if (!DSP_SUCCEEDED(status))
            {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Start failed.\n") ;
            }
    }

    return (status);
}


/*
 *  ======== InitializeDMMNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeDMMNode(DMMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA      * pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;

    uuid = Msguuid;

    nodeAttrIn.uTimeout = DSP_FOREVER;
    nodeAttrIn.iPriority = 5;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif



    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    pArgs->cData[0] = 0;

    /* Allocate the dmmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",status);
        }
    }

    /* Create the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
           TSTLOG_appendLog(pHandle,"DSPNode_Create failed. Status = 0x%x\n",status);
        }
    }

    /* Start the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n", status);
        }
    }

    return (status);
}


/*
 *  ======== RunDMMTask ========
 *  Run dmmcopy task.
 */
static DSP_STATUS RunDMMTask(DMMCOPY_TASK *copyTask,ULONG BufferSize)
{
    INT             cBytesRead = 0;
    DSP_STATUS      status = DSP_SOK;

    /* Actual MPU Buffer addresses */
    ULONG           aBufferSend = 0;

    /* Reserved buffer DSP virtual addresses (bytes) */
    ULONG           aDspSendBuffer = 0;

    /* Base addresses of reserved DSP virtual address ranges (bytes) */
    ULONG           dspAddrSend = 0;

    /* MPU buffer size */
    ULONG           ulSendBufferSize;

    /* DSP reserve size = buffersize + 4K page size */
    ULONG           ulSendResv;

    /* Messaging used for GPP/DSP synchronization */
    struct DSP_MSG         msgToDsp;
    struct DSP_MSG         msgFromDsp;

    INT             iCount = 0;
    struct          timeval *tvBeg;
    struct          timeval *tvEnd;
    struct          timeval *tvElapsed;
    struct          timezone *tz;
    INT             TimerResult=0;
     unsigned long long	    TimeInuSec = 0;
     unsigned long long     TimeInSec = 0;
     unsigned long long     ThroughPut = 0;
     unsigned long long     cTotalBytes = 0;



    tvBeg =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
    tvEnd =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
    tvElapsed =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
    tz =  (struct timezone*)OsSpecificMemAlloc(sizeof(struct timezone));


    /* Allocate MPU Buffers */
    ulSendBufferSize = BufferSize;

    ulSendResv = ulSendBufferSize + 0x1000;
 
    aBufferSend = (ULONG) malloc(ulSendBufferSize);

    /* Initialize buffers */
    for (iCount = 0; iCount < ulSendBufferSize; iCount++) {
        ((PSTR)aBufferSend)[iCount] = 0;
    }

    if (!aBufferSend ) {
        TSTLOG_appendLog(pHandle, "Failed to allocate MPU buffers.\n");
        status = DSP_EMEMORY;
    } else {

        /* Reserve DSP virtual memory for send buffer */
        status = DSPProcessor_ReserveMemory(copyTask->hProcessor,
                ulSendResv, (PVOID *)&dspAddrSend);

        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_ReserveMemory succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle,  "DSPProcessor_ReserveMemory failed. Status = 0x%x\n", status);
        }
    }
       switch(copyTask->AccessFlag)
        {
	/* No access to buffer */
                case 0:
                msgToDsp.dwCmd = 0xDCBA;
                break;
	/* Write access to buffer */
                case 1:
                msgToDsp.dwCmd = 0xDCBB;
                break;
	/* Read access to buffer */
                case 2:
                msgToDsp.dwCmd = 0xDCBC;
                break;
	/*Default*/
                default:
                TSTLOG_appendLog(pHandle, "Invalid Access Flag Value supplied Setting it to Default Value\n");
                msgToDsp.dwCmd = 0xDCBA;
                break;


        }
        msgToDsp.dwArg1 = (DWORD)BufferSize / g_dwDSPWordSize;
        msgToDsp.dwArg2 = (DWORD)aDspSendBuffer / g_dwDSPWordSize;



	TimerResult = gettimeofday(tvBeg,tz);
        if( TimerResult != 0)
         {
                TSTLOG_appendLog(pHandle, "GetTimeOfDay failed\n");
         }
/* Run DMM Copy for iCount times */

    for (iCount=0;iCount<copyTask->numIterations ; iCount++)

   {

    if (DSP_SUCCEEDED(status)) {

        /* Map MPU "send buffer" to DSP "receive buffer" virtual address */
        status = DSPProcessor_Map(copyTask->hProcessor,
                (PVOID)aBufferSend, ulSendBufferSize,
                (PVOID)dspAddrSend, (PVOID *)&aDspSendBuffer, 0);

        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Map failed.Status = 0x%x\n", status);
        }
    }

    if (DSP_SUCCEEDED(status)) {
                status = DSPProcessor_FlushMemory(copyTask->hProcessor, (PVOID)aBufferSend, ulSendBufferSize, 0);
		if (DSP_FAILED(status))
		{
          	   TSTLOG_appendLog(pHandle, "DSPProcessor_FlushMemory failed.Status = 0x%x\n", status);
		   break;
		}
}

    if (DSP_SUCCEEDED(status)) {

        /* Notify DSP of word-adjusted buffer addresses */
        	
        status = DSPNode_PutMessage(copyTask->hNode, &msgToDsp, DSP_FOREVER);
        if (DSP_FAILED(status)) {
          TSTLOG_appendLog(pHandle, "DSPProcessor_PutMessage failed.Status = 0x%x\n", status);
	  break;
        }

	if (msgToDsp.dwCmd != 0xDCBA)
	{
		status = DSPNode_GetMessage(copyTask->hNode,&msgFromDsp, DSP_FOREVER);
		if (DSP_FAILED(status)) {
          	TSTLOG_appendLog(pHandle, "DSPProcessor_GetMessage failed.Status = 0x%x\n", status);
	  	break;
		}
	}

    }

    	status = DSPProcessor_UnMap(copyTask->hProcessor,(PVOID)aDspSendBuffer);
	if(DSP_FAILED(status))
	{
          TSTLOG_appendLog(pHandle, "DSPProcessor_UnMap failed.Status = 0x%x\n", status);
	  break;
	}       
        if ((iCount % 500) == 0)
        {
            TSTLOG_appendLog(pHandle, "Map %d \n", iCount);
        }


}/*End of for(i=... */


    TimerResult = gettimeofday(tvEnd,tz);


    if( TimerResult != 0)
     {
            TSTLOG_appendLog(pHandle, "GetTimeOfDay failed\n");
     }

    if (tvBeg->tv_usec > tvEnd->tv_usec) {
       tvEnd->tv_usec += 1000000;
       tvEnd->tv_sec--;
     }

    TimeInuSec = (tvEnd->tv_sec - tvBeg->tv_sec)*1000000 + (tvEnd->tv_usec - tvBeg->tv_usec);
    TimeInSec  = (tvEnd->tv_sec - tvBeg->tv_sec) + (tvEnd->tv_usec - tvBeg->tv_usec)/1000000 ;
    cTotalBytes = copyTask->numIterations*BufferSize;
    TSTLOG_appendLog(pHandle, "Number of Total bytes 0x%x\n",  cTotalBytes);
    
    if(TimeInSec == 0)
	{
	
    		ThroughPut = (copyTask->numIterations/TimeInuSec)*BufferSize*1000*1000;
		/*Fix ME with correct data types*/
    		if(!ThroughPut)
		{
	 		ThroughPut = (copyTask->numIterations*BufferSize)/TimeInuSec*1000*1000;	
		}
	}
   	else
	{
		ThroughPut = (copyTask->numIterations/TimeInuSec)*BufferSize;
		/*Fix ME with correct data types*/
    		if(!ThroughPut)
		{
	 		ThroughPut = (copyTask->numIterations*BufferSize)/TimeInSec;	
		}
	}
                                    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
                                    TSTLOG_appendLog(pHandle, "LOG: *********DMM COPY STATISTICS**********\n");
				    if(DSP_SUCCEEDED(status))
				    {
                                    TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = PASS\n");
				    }
				    else
				    {
                                    TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
				    }
                                    TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", BufferSize);
                                    TSTLOG_appendLog(pHandle, "LOG: StartTime = %u\n", (tvBeg->tv_sec*1000000+tvBeg->tv_usec));
                                    TSTLOG_appendLog(pHandle, "LOG: EndTime = %u\n", (tvEnd->tv_sec*1000000+tvEnd->tv_usec));
                                    TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %u usec \n", TimeInuSec );			  
                                    TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Bytes/sec and 0x%x\n",ThroughPut,ThroughPut );
                                    TSTLOG_appendLog(pHandle, "LOG: ulNumBytesTransferred = %u\n", cTotalBytes);
                                    TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
                                    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");




    
    status = DSPProcessor_UnReserveMemory(copyTask->hProcessor,(PVOID)dspAddrSend);

    if (!DSP_SUCCEEDED(status)) {
        TSTLOG_appendLog(pHandle, "DSPProcessor_UnReserveMemory failed. Status = 0x%x\n", status);
    }
    /* Free MPU buffers (at originally allocated addresses) */
    if (aBufferSend) {
        free((VOID *)aBufferSend);
    }

    return (status);
}


/*
 *  ======== CleanupDMMNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupDMMNode(DMMCOPY_TASK *copyTask)
{
    DSP_STATUS      status;
    DSP_STATUS      exitStatus;

    if (copyTask->hNode) {

        /* Terminate DSP node */
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,  "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node */
        status = DSPNode_Delete(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        }
	else
	{
            TSTLOG_appendLog(pHandle, "DSPNode_Delete PASS: 0x%x\n", status);
	}

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupDMMProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS CleanupDMMProcessor(DMMCOPY_TASK *copyTask)
{
    DSP_STATUS      status;

    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (!DSP_SUCCEEDED(status)) {
           TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }
        copyTask->hProcessor = NULL;
    }

    return (status);
}



