/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge
*/
/*  ============================================================================
*   File    PutGetMsg.c
*
*   Path    $(TESTSUITEROOT)\Analysis\ZeroCopyMsg
*
*   Desc    Host Allocates a messgae buffer with data and send the message 
*	    with Zero Copy Command  DSP multiplies the MSG structure contents 
*	    by 2 and transfers it back to Host Received data is verified.
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   MAY 2 , 2006  Based on the Zero Copy Msg sample             Ramesh Gupta G.
*      Original Version.
*   ============================================================================
*/


#include <stdio.h>
#include <getopt_test.h>
#include <dbapi.h>
#include <stdlib.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>


static NODEARGS         NodeArgs;

/* Default stream attributes */
#define DEFAULTALIGNMENT   0
#define DEFAULTSEGMENT     0
#define DEFAULTTIMEOUT  50000000
#define DEFAULTBUFSIZESTR  "1024"        /* Default buffer size in bytes. */
#define DEFAULTDSPUNIT  0           /* Default unit number for the board. */
#define DEFAULTDSPEXEC  "msg_nortdx_ext_2430.dof64P"
#define DEFAULTNBUFS    1           /* Default # of buffers. */

#define ARGSIZE         32

/* strmcopy task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    struct DSP_UUID            hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE           hNode;          /* Handle to node. */
    BYTE                *pInBufs;     /* Input stream buffers. */
    BYTE                *pOutBufs;    /* Output stream buffers. */
    struct DSP_NOTIFICATION    hNotification;  /* DSP notification object. */
    INT                 nStrmMode;      /* stream mode */
} ZEROCOPY_TASK;

/*
 *  strmcopy node
 */

/* Forward declarations: */

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeProcessor_ZC(ZEROCOPY_TASK *copyTask);
static DSP_STATUS InitializeNode_ZC(ZEROCOPY_TASK *copyTask);
static DSP_STATUS CleanupProcessor_ZC(ZEROCOPY_TASK *copyTask);
static DSP_STATUS CleanupNode_ZC(ZEROCOPY_TASK *copyTask);
static DSP_STATUS RunTask_ZC(ZEROCOPY_TASK *copyTask);

struct DSP_MSG zcopymsgtodsp,zcopymsgfromdsp;
const int KDspBufDesc = 0x20000000;
DSP_STATUS MsgToDsp_ZC(ZEROCOPY_TASK *copyTask);
DSP_STATUS MsgFromDsp_ZC(ZEROCOPY_TASK *copyTask);
DSP_STATUS MsgToDsp_ThroughPut(ZEROCOPY_TASK *copyTask);
DSP_STATUS MsgFromDsp_ThroughPut(ZEROCOPY_TASK *copyTask);

DSP_STATUS MsgToDsp_ZC(ZEROCOPY_TASK *copyTask)
{
    int i = 0;
    DSP_STATUS     status    = DSP_SOK ;
    unsigned int * pSmMsgBuf = (int*) copyTask->pOutBufs ;

      
    if (DSP_FAILED(status)) 
    {
        TSTLOG_appendLog(pHandle, "DSPNode_AllocMsgBuf failed.\n");
    }

    zcopymsgtodsp.dwArg1 = (DWORD)pSmMsgBuf ;
    zcopymsgtodsp.dwCmd = DSP_RMSBUFDESC ;
    zcopymsgtodsp.dwArg2 = NodeArgs.iBufferSize;

    for (i = 0; i < (NodeArgs.iBufferSize / sizeof (unsigned int)) ; i++)
    {
         *pSmMsgBuf = (unsigned int)i ;
	 pSmMsgBuf++;
    }

    status = DSPNode_PutMessage(copyTask->hNode, &zcopymsgtodsp,DEFAULTTIMEOUT);
    if(status != DSP_SOK)
	{
    		TSTLOG_appendLog(pHandle, "PutMessage Failed and staus is 0x%x\n",status);
	}

    return status ;
}

DSP_STATUS MsgFromDsp_ZC(ZEROCOPY_TASK *copyTask)
{
    int i = 0;
    DSP_STATUS status = DSP_SOK ;
    unsigned int * dataRxAddr = NULL ;
    unsigned int * dspDataTx  = NULL ;
    int      msgSize    = 0 ;

    status = DSPNode_GetMessage(copyTask->hNode, &zcopymsgfromdsp,DEFAULTTIMEOUT);
    if(status != DSP_SOK)
	{
    		TSTLOG_appendLog(pHandle, "GetMessage Failed and staus is 0x%x\n",status);
	}

    msgSize = zcopymsgfromdsp.dwArg2 ;


    if ((status         !=  DSP_SOK) ||
        (msgSize        ==  0)        ||
        (zcopymsgfromdsp.dwCmd !=  (int) DSP_RMSBUFDESC)) //  not a zero-copy message
    {
        return status ;
    }

    dataRxAddr = (unsigned int *)zcopymsgfromdsp.dwArg1 ;
    dspDataTx = (unsigned int *) zcopymsgtodsp.dwArg1 ;
    copyTask->pInBufs = (BYTE *)zcopymsgfromdsp.dwArg1 ;

    for (i = 0; i < (NodeArgs.iBufferSize / sizeof (unsigned int)) ; i++)
    {
       if (dataRxAddr[i] != (2*dspDataTx[i]))
       {
          TSTLOG_appendLog(pHandle, " Error Sent = %d Received = %d and buffer index is %d ",(unsigned int) dspDataTx[i], (unsigned int)dataRxAddr[i],i) ;
          status = DSP_EFAIL;
          break;
       }
	
    }

    return status ;
}



DSP_STATUS MsgToDsp_ThroughPut(ZEROCOPY_TASK *copyTask)
{
    int i = 0;
    DSP_STATUS     status    = DSP_SOK ;
    unsigned int * pSmMsgBuf = (int*) copyTask->pOutBufs ;


    if (DSP_FAILED(status))
    {
        TSTLOG_appendLog(pHandle, "DSPNode_AllocMsgBuf failed.\n");
    }

    zcopymsgtodsp.dwArg1 = (DWORD)pSmMsgBuf ;
    zcopymsgtodsp.dwCmd = DSP_RMSBUFDESC ;
    zcopymsgtodsp.dwArg2 = NodeArgs.iBufferSize;

    status = DSPNode_PutMessage(copyTask->hNode, &zcopymsgtodsp,DEFAULTTIMEOUT);
    if(status != DSP_SOK)
        {
                TSTLOG_appendLog(pHandle, "PutMessage Failed and staus is 0x%x\n",status);
        }

    return status ;
}


DSP_STATUS MsgFromDsp_ThroughPut(ZEROCOPY_TASK *copyTask)
{
    int i = 0;
    DSP_STATUS status = DSP_SOK ;
    unsigned int * dataRxAddr = NULL ;
    unsigned int * dspDataTx  = NULL ;
    int      msgSize    = 0 ;

    status = DSPNode_GetMessage(copyTask->hNode, &zcopymsgfromdsp,DEFAULTTIMEOUT);
    if(status != DSP_SOK)
        {
                TSTLOG_appendLog(pHandle, "GetMessage Failed and staus is 0x%x\n",status);
        }

    return status ;
}



/*
 *  ======== main ========
 */
int zeroCopyMsg_Test(int argc, char ** argv)
{
    INT             nStrmMode = STRMMODE_ZEROCOPY;
    ZEROCOPY_TASK   strmcopyTask;
    DSP_STATUS      status = DSP_SOK;

    INT             opt = 0;
    NodeArgs.iSegmentId = DEFAULTSEGMENT;
    NodeArgs.iMode  = 0;
    optind = 0;
        while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:b:n:ZPDR:")) != EOF) {

        switch (opt) {
              case 'v':
                        /* verbose */
                        TST_setVerbose(TRUE);
                        break;

              case 'l':
            /* use log and store with name location */
                        strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
                        break;

             case 'x':
            /* use given executable */
                        aArgv[0] = (char *) optarg;
                        fLoadDsp = TRUE;
                        break;

             case 'b':
            /* set Buffer Size */
                        NodeArgs.iBufferSize = CSL_Atoi(optarg);
                        break;

             case 'n':
            /* set Number of transfers */
                        NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
                        break;

            default:
                        TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
                        break;
        }
    }



    DspManager_Open(argc, NULL);



    strmcopyTask.hProcessor = NULL;
    strmcopyTask.hNode = NULL;

    strmcopyTask.pInBufs = NULL;
    strmcopyTask.pOutBufs = NULL;
    strmcopyTask.nStrmMode = nStrmMode;
    if (DSP_SUCCEEDED(status)) {
        /* Perform processor level initialization. */
        status = InitializeProcessor_ZC(&strmcopyTask);
    }

    if (DSP_SUCCEEDED(status)) {

        /* Perform node level initialization. */
        status = InitializeNode_ZC(&strmcopyTask);
        if (DSP_SUCCEEDED(status)) {

                /* Run task. */
                status = RunTask_ZC(&strmcopyTask);
                if (DSP_SUCCEEDED(status)) {
                    TSTLOG_appendLog(pHandle, "RunTask succeeded.\n");
                    TSTLOG_appendLog(pHandle, "Zecopy Message successful\n");
                }
                else {
                    TSTLOG_appendLog(pHandle, "RunTask failed.\n");
                    TSTLOG_appendLog(pHandle, "Zecopy Message failed\n");
                }
        }
    }

    /* Close opened files. */

    CleanupNode_ZC(&strmcopyTask);
    CleanupProcessor_ZC(&strmcopyTask);

    DspManager_Close(0,NULL);

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor_ZC(ZEROCOPY_TASK *copyTask)
{  
    DSP_STATUS      status = DSP_EFAIL;
    struct DSP_PROCESSORINFO	dspInfo;
    UINT				numProcs;
    UINT				index=0;
    INT					procId=0;
    INT                                 iArgc = 1;
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,
                    &dspInfo,
                    (UINT) sizeof (struct DSP_PROCESSORINFO),
                    &numProcs)))
    {
        if ((dspInfo.uProcessorType == DSPTYPE_55) || (dspInfo.uProcessorType == DSPTYPE_64))
        {
            printf("DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;	
            break;	
        }
        index++;	
    }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&copyTask->hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",
                    (UINT)status);
        }
    }
    else
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

    if (DSP_SUCCEEDED(status))
    {
        TSTLOG_appendLog(pHandle, " DSP Image: %s.\n", aArgv[0]) ;

        status = DSPProcessor_Load(copyTask->hProcessor,iArgc,&aArgv,NULL) ;

        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded.\n") ;
        }
        else
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load failed.\n") ;
        }

        status = DSPProcessor_Start(copyTask->hProcessor) ;

        if (DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Start succeeded.\n") ;
        }
        else
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Start failed.\n") ;
        }
    }
    return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode_ZC(ZEROCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;

    uuid = Msguuid;

    attrs.uBufsize = DEFAULTBUFSIZE;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = copyTask->nStrmMode;
    if (copyTask->nStrmMode == STRMMODE_ZEROCOPY) {
        attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }
    else 
    {
        attrs.uSegid = DEFAULTSEGMENT;
    }

    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif


    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    strncpy(pArgs->cData,DEFAULTBUFSIZESTR, 5); 

    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",(UINT)status);
	    return status;
        }
    }

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n",(UINT)status);
	    return status;
        }
    }

    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",(UINT)status);
	    return status;
        }
    }

    return (status);
}

/*
 *  ======== RunTask ========
 *  Run strmcopy task.
 */
static DSP_STATUS RunTask_ZC(ZEROCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    BYTE * 	    pSmMsgBuf = NULL;
    UINT            k;
    DWORD           dwStartTime;
    DWORD           dwElapsedTime;
    DWORD           dwEndTime;
    DWORD           ulNumBytesTransferred = 0;
    DWORD           ulBytesPerSec = 0;
    INT             i = 0;

    //  Allocate zero-copy message buffer
    status = DSPNode_AllocMsgBuf(copyTask->hNode,NodeArgs.iBufferSize, NULL,(BYTE **)&pSmMsgBuf);


    if (DSP_FAILED(status))
    {
        TSTLOG_appendLog(pHandle, "DSPNode_AllocMsgBuf failed and status = 0x%x.\n",status);
    }

    copyTask->pOutBufs = pSmMsgBuf;

    // Put a buffer into the output channel.
    for(k = 0; k < 100; k++)
    {

    status = MsgToDsp_ZC(copyTask);
    if (!DSP_SUCCEEDED(status)) {
               TSTLOG_appendLog(pHandle, "MsgToDsp failed.\n");
               status = DSP_EFAIL;
          }
    status = MsgFromDsp_ZC(copyTask);
    if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "MsgFromDsp failed.\n");
            status = DSP_EFAIL;
          }
    //  Free zero-copy message buffer
    }


    zcopymsgtodsp.dwArg1 = (DWORD)pSmMsgBuf ;
    zcopymsgtodsp.dwCmd = RMS_USER + 18 ;
    zcopymsgtodsp.dwArg2 = NodeArgs.iBufferSize;

    status = DSPNode_PutMessage(copyTask->hNode, &zcopymsgtodsp,DEFAULTTIMEOUT);
    if(status != DSP_SOK)
        {
                TSTLOG_appendLog(pHandle, "PutMessage Failed and staus is 0x%x\n",status);
        }

 OsSpecificTimerStart(&dwStartTime);

    for(k = 0; k < NodeArgs.iNumTxfrs; k++)
    {

    status = MsgToDsp_ThroughPut(copyTask);
    if (!DSP_SUCCEEDED(status)) {
               TSTLOG_appendLog(pHandle, "MsgToDsp failed.\n");
               status = DSP_EFAIL;
          }
    status = MsgFromDsp_ThroughPut(copyTask);
    if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "MsgFromDsp failed.\n");
            status = DSP_EFAIL;
          }
    //  Free zero-copy message buffer
    }

    OsSpecificTimerStop(&dwEndTime);
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);

    status = DSPNode_FreeMsgBuf(copyTask->hNode, pSmMsgBuf,NULL);
    TSTLOG_appendLog(pHandle,"DSPNode_FreeMsgBuf pSmMsgBuf:%x\n",(unsigned int)pSmMsgBuf);
    if(status != DSP_SOK)
    {
            TSTLOG_appendLog(pHandle, "DSPNode_FreeMsgBuf Failed and  STATUS = 0x%x\n", status);

    }
	

    if (DSP_SUCCEEDED(status))
    {
        TSTLOG_appendLog(pHandle,"Number Of Iterations is %d\n",NodeArgs.iNumTxfrs);

        ulNumBytesTransferred = (NodeArgs.iNumTxfrs * NodeArgs.iBufferSize);
        ulBytesPerSec = (ulNumBytesTransferred/dwElapsedTime)*1000;
        if(fOpenLog)
        {
            TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
            TSTLOG_appendLog(pHandle, "LOG: *********ZEROCOPY MSG STATISTICS**********\n");
            TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = PASS\n");
            TSTLOG_appendLog(pHandle, "LOG: BufferSize = 0x%d\n",NodeArgs.iBufferSize);
            TSTLOG_appendLog(pHandle, "LOG: StartTime = %d\n", dwStartTime);
            TSTLOG_appendLog(pHandle, "LOG: EndTime = %d\n", dwEndTime);
            TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %d\n", dwElapsedTime);
            TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Bytes/sec\n", ulBytesPerSec);
            TSTLOG_appendLog(pHandle, "LOG: Number of Transfers = %d\n", NodeArgs.iNumTxfrs);
            TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
            TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
        }
    }
    else
    {
        if(fOpenLog)
        {
            TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
        }
        TST_verbose("status = 0x%x \n",status);
    }




    return (status);
}

/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupNode_ZC(ZEROCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus;

    if (copyTask->hNode) {

        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed: 0x%x\n", (UINT)status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", (UINT)status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS CleanupProcessor_ZC(ZEROCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}
