/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    waitforevent.h
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\Include
 * 
 *   Desc:   Header file for Wait for Events Test Case
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *   OCT 05, 2005   updated for 2430     Ramesh Gupta G 
 *   NOV 14, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#define NUMNOTIFOBJECTS     4             /* Number of Notification Objects */
#define DEFNAMESIZE         80
#define NUMOFCOMPARGS       0
#define DEFAULTDSPUNIT      0
#define ARGSIZE             32

/* Default stream attributes */
#define DEFAULTALIGNMENT    0
#define DEFAULTSEGMENT      0
#define DEFAULTTIMEOUT      5000
#define DEFAULTBUFSIZE      1024            /* Default buffer size in bytes. */
#define DEFAULTNBUFS        1               /* Default # of buffers. */

#define PASS 0
#define FAIL 1

#define ONEEVENT         1
#define TWOEVENTS        2
#define THREEEVENTS      3
#define FOUREVENTS       4


#ifdef OMAP_2430
    #define DSPEXEC "./msg_nortdx_int_2430.dof64P"   /* DSP Executable name */
    #define STRMDSPEXEC "./inout_nortdx_int_2430.dof64P"   /* DSP strm Executable name */
#else
    #define DSPEXEC "./msg_nortdx_int_24xx.dof55L"   /* DSP Executable name */
    #define STRMDSPEXEC "./inout_nortdx_int_24xx.dof55L"   /* DSP strm Executable name */
#endif


/* task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    struct DSP_UUID            hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE           hNode;          /* Handle to node. */
    DSP_HSTREAM         hInStream;      /* Handle to node input stream. */
    DSP_HSTREAM         hOutStream;     /* Handle to node output stream. */
    BYTE                **ppInBufs;     /* Input stream buffers. */
    BYTE                **ppOutBufs;    /* Output stream buffers. */
    struct DSP_NOTIFICATION    hNotification;  /* DSP notification object. */
    INT                 nStrmMode;      /* stream mode */
    UINT                procEventMask;  /* Event Mask for Processor */
    UINT                nodeEventMask;  /* Event Mask for Node*/
    UINT                strmEventMask;  /* Event Mask for Stream */
    UINT                notifyMask;     /* Notify mask currently same for Node and Proc */
    struct DSP_NOTIFICATION*   *notifications; /* Pointer to array of Notification Objects */
    struct DSP_NOTIFICATION*   *strmnotifications; /* Pointer to array of Notification Objects */
    FILE                *pHandle; 		/* For log file handle */
} TASK_CONTEXT;


/* Initialization and cleanup routines */
static DSP_STATUS checkProcessorEvents(TASK_CONTEXT *copyTask,CHAR *aArgv);
static DSP_STATUS checkNodeEvents(TASK_CONTEXT *copyTask);
static DSP_STATUS checkStreamEvents(TASK_CONTEXT *copyTask,CHAR *strmArgv);
static DSP_STATUS CleanupProcessor(TASK_CONTEXT *copyTask);
static DSP_STATUS CleanupNode(TASK_CONTEXT *copyTask);
static DSP_STATUS CleanupStreams(TASK_CONTEXT *copyTask);



