/*  ============================================================================
 *   File    IVAMultipleAttach.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\ProcAttach
 * 
 *   Desc:   Tests the Behaviour of the Bridge for Multiple
 *           Processor Attach and Detach calls.
 *
 *  Usage:
 *        IVAMULATTACH [-v] [-l "log path/filename"] 
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -n: Number of times Attach needs to be done !
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG                    -Swamy MC
 *   JULY 22, 2004   Modified to work for IVA   -RAMESH GUPTA G
 *      Original Version.
 *   ============================================================================
 */


/*
 *  ======== .c ========
 *
 *
 *! Revision History
 *! ================
 *! 21-Aug-2002 swa: Created
 */

#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#include "ProcAttach.h"

#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      1	/* Compulsory args: t */
#define MAXNUMATTACHES    10    /* Max num of Attaches that can be done */
#define PASS               0
#define FAIL               1


/*
 *  ======== main function ========
 *
 */
INT IVAmultipleAttach_BEHAVIOUR(INT argc, CHAR * argv[])
{
        BOOL                    bUseLog=FALSE;
        FILE                    *pHandle = NULL;
	CHAR                    szLogName[DEFNAMESIZE];
	INT                     iMandArgCount = NUMOFCOMPARGS;
	INT                     iNumAttaches =0;
	DSP_STATUS              status;
	INT                     opt;
	INT                     count;
        INT                     retVal = PASS;
 	INT                 index=0;
    	INT                 procId;
    	struct DSP_PROCESSORINFO   dspInfo;
    	UINT                numProcs;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor[MAXNUMATTACHES];
        struct DSP_PROCESSORATTRIN		*pAttrIn;


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Attach to the Processor requested number of times */
	/* End !! */
	/* Process arguments and Store the values */
        optind = 0;
	opt = getopt_test(argc, (char *const)argv, "+vl:n:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
		case 'n':
			/* Number of Threads to be created */
			iNumAttaches = CSL_Atoi(optarg);
			iMandArgCount--;
			break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char *const)argv, "+vl:n:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		return FAIL;
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

////////////////////////////////////


    /* Attach to an available DSP (in this case, the IVA). */


  while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }


////////////////////////////////////
	for (count = 0; count < iNumAttaches; count ++)
	{
		/* Attach IVA  to GPP Processor */
		status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor[count]);

		if (DSP_FAILED(status)) 
		{
	        /* Report Error */
			if (bUseLog) 
			{
				TSTLOG_appendLog(pHandle, "Error in DSPProcessor_Attach():IVA %d\n",count);
			}
			retVal = FAIL;
			break;
		}
		else 
		{
			if (bUseLog) 
			{
				TSTLOG_appendLog(pHandle, "DSPProcessor_Attach() Passed in Iteration %d\n",count);
			}
		}
	}

	if (retVal != FAIL)
	{
		for (count = 0; count < iNumAttaches; count ++)
		{
			DSPProcessor_Detach(hProcessor[count]);
		}
	}

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

