/*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge
 */
/*  ============================================================================
 *   File    MultiRetRun.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\PwrMgmt\
 *
 *   Desc:   Multiple RETENTION/RUN test case
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   May 10, 2006    REF=ORG             Ramesh Gupta G
 *      Original Version.
 *   ============================================================================
 */


#include <stdlib.h>
#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>

#include <fcntl.h>
#include <sys/mman.h>

#include <pwr_sh.h>
#include <unistd.h>
//PRCM area size
#define PRCM_SIZE 0x1000
#define INACTIVITYTIMER 30

#define ARGSIZE       32            /* Size of arguments to Ping node.       */

static NODEARGS            NodeArgs;

static void ChkForPowerState(unsigned long  prcm_base);
static DSP_STATUS SendMsg(COPY_TASK *copyTask,struct DSP_MSG * msg,UINT uTimeout);


/*
*  ======== multiNodeRun_BEHAVIOUR ========
*/
INT multiNodeRun_BEHAVIOUR(INT argc, CHAR ** argv)
{
    COPY_TASK       	copyTask;
    DSP_STATUS      	status = DSP_SOK;
    INT             	opt = 0;
    struct DSP_PROCESSORINFO   dspInfo;
    UINT                numProcs;
    UINT                index=0;
    INT                 procId=0;
    INT 		numIterations = 0;
    fLoadDsp = FALSE;
    copyTask.hProcessor = NULL;
    copyTask.hNode = NULL;
    copyTask.hInStream = NULL;

        optind = 0;
          while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:i:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

              case 'l':
            /* use log and store with name location */
            		strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
            		TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
                        break;
              case 'x':
            /* use given executable */
            		aArgv[0] = (char *) optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
                        break;

                case 'n':
            /* set Number of Iterations */
                        NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Number of Data Transfers =  %i\n", NodeArgs.iNumTxfrs);
                        break;

                case 'i':
                        /* set Number of Iterations */
			numIterations = CSL_Atoi(optarg);
                        NodeArgs.iMode = STRMMODE_ZEROCOPY;
                        break;
                case 'R':
                        /* set Transfer Mode as STRMMODE_RDMA */
                        NodeArgs.iMode = STRMMODE_RDMA;
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Transfer Mode Set to =  %i\n", STRMMODE_RDMA);
                        break;

        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }

    /* Attach to processor */
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
        if (DSP_SUCCEEDED(status))
        {
               /* Load user specified DSP Executable and start the processor. */
                if(fLoadDsp)
                status = InitializeProcessor(&copyTask);
        }
    if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
        status = InitializeNode(&copyTask, &NodeArgs);
        if (DSP_SUCCEEDED(status))
                {

			for(index=0;index < numIterations ; index++)
			{
                       		 status = RunTask(&copyTask, &NodeArgs);
                        	 if (!DSP_SUCCEEDED(status))
                        	 {
                                	if(fOpenLog)
                                        TSTLOG_appendLog(pHandle,  "RunTask Failed. Iteration = %d Status = 0x%x\n",numIterations,status);
                        	 }
			}
        }
    }

    CleanupNode(&copyTask);
    if (copyTask.hProcessor)
        {
        DSPProcessor_Detach(copyTask.hProcessor);
        copyTask.hProcessor = NULL;
    }
    status = DspManager_Close(0,NULL) ;
    return (DSP_SUCCEEDED(status) ? 0 : -1);

}


/*
*  ======== InitializeProcessor ========
*  Perform processor related initialization.
*/
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_EFAIL;
    INT                 iArgc = 1;
    
        /* Load user specified DSP Executable and start the processor. */
    
   	 status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp);  
    	if (DSP_FAILED(status)) 
    	{
        	if(fOpenLog)
            	TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = 0x%x\n", status);
    	}
    	else
    	{
        	status = DSPProcessor_Start(copyTask->hProcessor);
        	if (DSP_FAILED(status)) 
        	{
            	if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start. STATUS = 0x%x\n", status);
        	}
        }
    
    return (status);
}


/*
*  ======== InitializeNode ========
*  Perform node related initialization.
*/
static DSP_STATUS InitializeNode(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;

    uuid = Msguuid;

    nodeAttrIn.uTimeout = 1000;
    nodeAttrIn.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif


    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    pArgs->cData[0] = 0;

    /* Allocate the message node. */
    status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
    if (DSP_SUCCEEDED(status))
        {
                /* Create the msg node on the DSP. */
                status = DSPNode_Create(copyTask->hNode);
                if (DSP_SUCCEEDED(status))
                {
                        /* Start the msg node on the DSP. */
                        status = DSPNode_Run(copyTask->hNode);
                        if (DSP_FAILED(status))
                        {
                                if(fOpenLog)
                                        TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
                        }
                }
                else
                {
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle,"DSPNode_Create failed. Status = 0x%x\n",status);
                }
    }
    else
        {
           if(fOpenLog)
                        TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",status);
    }
        return (status);

}


/*
*  ======== RunTask ========
*  Run copy task.
*/
static DSP_STATUS RunTask(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    DSP_STATUS          status;
    struct DSP_MSG             msg;
    UINT 		uTimeout = 1000;
    unsigned long   tmp_adr;                            
    unsigned long   prcm_base=0;
    unsigned long   *prcm_adr;                          
    int             fd;
    char            *PRCM_PHYSADR  =(char *) 0x49006000;

    
    fd = open("/dev/mem", O_RDWR|O_SYNC);
    
    if (fd) {
        prcm_base = (UINT)(char *) mmap ( 0,
            PRCM_SIZE * sizeof(char),
            PROT_READ | PROT_WRITE,
            MAP_SHARED,
            fd,
            (unsigned long)PRCM_PHYSADR);
        
        TSTLOG_appendLog(pHandle, "mapped prcm_base VA=0x%x , PA=0x%x \n", (UINT)prcm_base,0x49006000);

        close (fd);
    }

	    SendMsg(copyTask,&msg,uTimeout);
            tmp_adr = prcm_base + 0x8E4;
            prcm_adr = (unsigned long *)tmp_adr;
            TSTLOG_appendLog(pHandle, "*** DSP domain POWER state after sending Messages ***\n");
	    ChkForPowerState(prcm_adr);

            
            sleep(INACTIVITYTIMER);
            TSTLOG_appendLog(pHandle, "*** DSP domain POWER state after Sleep ***\n");
	    ChkForPowerState(prcm_adr);
	    SendMsg(copyTask,&msg,uTimeout);
	   

            TSTLOG_appendLog(pHandle, "*** DSP domain POWER state after Sending Message when DSP in sleep ***\n");
	    ChkForPowerState(prcm_adr);
            /* send Power DEEPSLEEP command to IVA2 */        
            status = DSPProcessor_Ctrl(copyTask->hProcessor, PWR_DEEPSLEEP, NULL);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PWR_DEEPSLEEP succeeded.\n");
            }    
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PWR_DEEPSLEEP failed. status: 0x%x\n", (UINT)status);
            }

            sleep(2); // wait to DSP to go IDLE mode
            TSTLOG_appendLog(pHandle,"*** DSP domain POWER state after Sending DEEPSLEEP Ctrl Message ***\n");
	    ChkForPowerState(prcm_adr);

#ifdef DISABLE_BRIDGE_PM

	    status = DSPProcessor_Ctrl(copyTask->hProcessor, PWR_WAKEUP, NULL);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PWR_WAKEUP succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Ctrl:PWR_WAKEUP failed. status: 0x%x\n", (UINT)status);
            }

#endif
	    SendMsg(copyTask,&msg,uTimeout);
	    ChkForPowerState(prcm_adr);
    	    munmap((char *)prcm_base, PRCM_SIZE * sizeof(char));
    
    return (status);
}

/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupNode(COPY_TASK *copyTask)
{
    DSP_STATUS      status  = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    if (copyTask->hNode) {

                /* Terminate the node */
                status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
                if (DSP_FAILED(status))
                {
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle,  "DSPNode_Terminate failed.status = 0x%x\n", status);
                }
		else
		{
        /* Delete DSP node. */
        	status = DSPNode_Delete(copyTask->hNode);
        		if (DSP_FAILED(status))
        		{
            		if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "DSPNode_Delete failed.\n");
        		}
		}

        copyTask->hNode = NULL;
    }

    return (status);
}

static void ChkForPowerState(unsigned long  prcm_adr)
{

            switch ( *(( volatile unsigned long * )prcm_adr ) & 0x00000003 )
            {
            case 0x0:
                TSTLOG_appendLog(pHandle,"DSP power domain status is ON\n");
                break;
            case 0x1:
                TSTLOG_appendLog(pHandle,"DSP power domain status is RETENTION\n");
                break;
            case 0x2:
                TSTLOG_appendLog(pHandle,"DSP power domain status bit is Reserved\n");
                break;
            case 0x3:
                TSTLOG_appendLog(pHandle,"DSP power domain status is OFF\n");
                break;
            default:
                TSTLOG_appendLog(pHandle,"DSP power domain status is UNKNOWN\n");
                break;
            }


}

static DSP_STATUS SendMsg(COPY_TASK *copyTask,struct DSP_MSG * msg,UINT uTimeout)
{
	int 	   k = 0;
	DSP_STATUS status=DSP_SOK;


                for(k = 0; k < NodeArgs.iNumTxfrs/2; k++)
                {
                                /* Set msg SM buffer descriptor cmd code. */
                        msg->dwCmd = RMS_USER + 3;  //CMD_REPLY. -> multiply dwArg1 by 2.
                        msg->dwArg1 = k;

                        status = DSPNode_PutMessage(copyTask->hNode, msg, uTimeout);
                        sleep(2);
                        if (DSP_FAILED(status))
                        {
                        if(fOpenLog)
                        TSTLOG_appendLog(pHandle, "DSPNode_PutMessage failed. Status = 0x%x\n",status);
                        break;
                        }

                        status = DSPNode_GetMessage(copyTask->hNode, msg, uTimeout);
                        /*This DSP executable multiplies dwArg1 by 2*/
                        if (DSP_SUCCEEDED(status))
                        {
                         if(msg->dwArg1 == (k*2))
                          {
                                if ((k % 5) == 0)
                                {
                                TST_verbose("PUTGETMSG  %d\n",k) ;
                                }
                          }
                          else
                          {
                                if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Processor Copy Message Data Verification failed\n");
                                break;

                          }
                        }
                        else
                        {
                                if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "DSPNode_GetMessage Failed. Status = 0x%x \n",status);
                                break;
                        }
                }

return (status);

}
