/*
 *  Copyright 2006 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge
 */
/*  ============================================================================
 *   File    MultiStartStop.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\PwrMgmt\
 *
 *   Desc:   Multiple START/RETENTION/STOP test case
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   May 10, 2006    REF=ORG             Ramesh Gupta G
 *      Original Version.
 *   ============================================================================
 */


#include <stdlib.h>
#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>

#include <fcntl.h>
#include <sys/mman.h>

#include <pwr_sh.h>
#include <unistd.h>
//PRCM area size
#define PRCM_SIZE 0x1000
#define INACTIVITYTIMER 30

#define ARGSIZE       32            /* Size of arguments to Ping node.       */


static void ChkForPowerState(unsigned long  prcm_base);
static DSP_STATUS StopProcessor(COPY_TASK *copyTask);


/*
*  ======== multiStartRetStop_BEHAVIOUR ========
*/
INT  multiStartRetStop_BEHAVIOUR(INT argc, CHAR ** argv)
{
    COPY_TASK       	copyTask;
    DSP_STATUS      	status = DSP_SOK;
    INT             	opt = 0;
    struct DSP_PROCESSORINFO   dspInfo;
    UINT                numProcs;
    UINT                index=0;
    INT                 procId=0;
    INT 		numIterations = 0;
    INT             	fd;
    fLoadDsp = FALSE;

    copyTask.hProcessor = NULL;
    copyTask.hNode = NULL;
    copyTask.hInStream = NULL;
    unsigned long   	tmp_adr;
    unsigned long   	prcm_base=0;
    unsigned long   	*prcm_adr;
    char            *PRCM_PHYSADR  =(char *) 0x49006000;



        optind = 0;
          while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

              case 'l':
            /* use log and store with name location */
            		strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
            		TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
                        break;
              case 'x':
            /* use given executable */
            		aArgv[0] = (char *) optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
                        break;

                case 'n':
            /* set Number of Iterations */
                        numIterations = CSL_Atoi(optarg);
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Number of Data Iterations =  %d\n", numIterations);
                        break;


        	default:
            		TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            		break;
        }
    }

    /* Attach to processor */
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
        if (DSP_SUCCEEDED(status))
        {
    		fd = open("/dev/mem", O_RDWR|O_SYNC);
        	if (fd) {
        	prcm_base = (UINT)(char *) mmap ( 0,
            	PRCM_SIZE * sizeof(char),
            	PROT_READ | PROT_WRITE,
            	MAP_SHARED,
            	fd,
            	(unsigned long)PRCM_PHYSADR);

        TSTLOG_appendLog(pHandle, "mapped prcm_base VA=0x%x , PA=0x%x \n", (UINT)prcm_base,0x49006000);

        	close (fd);
    }


	    tmp_adr = prcm_base + 0x8E4;
            prcm_adr = (unsigned long *)tmp_adr;

               /* Load user specified DSP Executable and start the processor. */
		for (index=0; index<numIterations ; index++)
		{

                	status = InitializeProcessor(&copyTask);
                        TSTLOG_appendLog(pHandle, "DSP domain state after Initalizing Processor.\n");
            		ChkForPowerState(prcm_adr);
			sleep(INACTIVITYTIMER);
                        TSTLOG_appendLog(pHandle, "DSP domain state after Sleep .\n");
            		ChkForPowerState(prcm_adr);
			status = StopProcessor(&copyTask);
                        TSTLOG_appendLog(pHandle, "DSP domain state after Stopping the Processor .\n");
            		ChkForPowerState(prcm_adr);
		}
        }


          /* Detach from processor. */
          status = DSPProcessor_Detach(copyTask.hProcessor);
                if (DSP_FAILED(status))
                {
                        if(fOpenLog)
                        TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
                }

    status = DspManager_Close(0,NULL) ;
    return (DSP_SUCCEEDED(status) ? 0 : -1);

}


/*
*  ======== InitializeProcessor ========
*  Perform processor related initialization.
*/
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_EFAIL;
    INT                 iArgc = 1;
    
        /* Load user specified DSP Executable and start the processor. */
    
   	 status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp);  
    	if (DSP_FAILED(status)) 
    	{
        	if(fOpenLog)
            	TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = 0x%x\n", status);
    	}
    	else
    	{
        	status = DSPProcessor_Start(copyTask->hProcessor);
        	if (DSP_FAILED(status)) 
        	{
            	if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start. STATUS = 0x%x\n", status);
        	}
        }
    
    return (status);
}


/*
 *  ======== Stop Processor ========
 *  Stops the DSP Processor.
 */
static DSP_STATUS StopProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

	/* Stop the Processor */
	status = DSPProcessor_Stop(copyTask->hProcessor);
	if(DSP_FAILED(status))
        {
                        if(fOpenLog)
                        TSTLOG_appendLog(pHandle, "DSPProcessor_Stop failed.\n");
        }
    return (status);
}


static void ChkForPowerState(unsigned long  prcm_adr)
{

            switch ( *(( volatile unsigned long * )prcm_adr ) & 0x00000003 )
            {
            case 0x0:
                TSTLOG_appendLog(pHandle,"DSP power domain status is ON\n");
                break;
            case 0x1:
                TSTLOG_appendLog(pHandle,"DSP power domain status is RETENTION\n");
                break;
            case 0x2:
                TSTLOG_appendLog(pHandle,"DSP power domain status bit is Reserved\n");
                break;
            case 0x3:
                TSTLOG_appendLog(pHandle,"DSP power domain status is OFF\n");
                break;
            default:
                TSTLOG_appendLog(pHandle,"DSP power domain status is UNKNOWN\n");
                break;
            }

}

