/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *  ============================================================================
 *   File    time.c
 *
 *   Path    $(TESTSUITEROOT)\OSAL
 *
 *   Desc:
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *!   ============================================================================
 *!   Revision History
 *!
 *!   NOV xx, 2002    REF=ORG             Raghu
 *!      Original Version.
 *!   ============================================================================
 */

#include <testosal.h>

void OsSpecificTimerStart(DWORD *dwStartTime)
{
#ifndef __KERNEL__
	struct timeb ts;
	ftime(&ts);
	*dwStartTime = ((ts.time * 1000) + ts.millitm);
#else
    struct timeval tv;
    do_gettimeofday(&tv);
    *dwStartTime = ((tv.tv_sec * 1000) + (tv.tv_usec / 1000));
#endif
}

void OsSpecificTimerStop(DWORD *dwEndTime)
{
#ifndef __KERNEL__
	struct timeb ts;
	ftime(&ts);
	*dwEndTime = ((ts.time * 1000) + ts.millitm);
#else
    struct timeval tv;
    do_gettimeofday(&tv);
    *dwEndTime = ((tv.tv_sec * 1000) + (tv.tv_usec / 1000));
#endif
}

void OsSpecificTimerDifference(DWORD *dwEndTime, DWORD *dwStartTime, DWORD *dwElapsedTime)
{
	*dwElapsedTime = *dwEndTime - *dwStartTime;

    /* This is an attempt to mask an imprecise calculation where dwEndTime
     * may be equal to dwStartTime.  We do not want to return 0 here to
     * break many tests. However, OS timer may be imprecise. */
    if (*dwElapsedTime == 0) {
        *dwElapsedTime = 1;
    }
}

void OsSpecificDelay(int iSecsToSleep)
{
#ifndef __KERNEL__
	sleep(iSecsToSleep);
#endif
}
