/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DynCircDep.c
 *
 *   Path    $(TESTSUITEROOT)/Stress/DynLink/
 * 
 *   Desc:
 *           This test case tests Dependant Library support of 
 *			 dynamic loading/linking Feature.
 *           The program stresses the bridge with two dep libs
 *			 which are circularly dependent on each other. 
 *
 *  Usage:
 *        DYNCIRCDEP [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 22, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <semaphore.h>
#include <DSPProcessor_OEM.h>
#include "dynLink.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define NUMOFCOMPARGS      2	/* Compulsory args: n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC         "baseimage_nortdxidl.dof"
#define DLLNAME         "/release/inoutdyncircdepstress.dll55l"
static sem_t semCreate = { {0,0}, 1 };


/*static DSP_STATUS SendTestConfiguration(SHORT **pOutBuf, 
										DSP_HSTREAM hOutStream, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize,
										UINT iNumTransfers);*/

/* Function to exit gracefully */
static INT graceful_exit()
{
	TST_verbose("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);
}

/* Function for Dep Lib check test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents. Copy the contents to local variables    *
 * and use	*
 */
static INT dynCircDepLibcheckFxn(dynLinkArgs * targs)
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	/*DSP_HSTREAM				hStream;*/
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	/*INT						i,k;
	DSP_STATUS				exitStatus = DSP_SOK;
	DSP_STREAMATTRIN        strmAttrs;*/

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	 BYTE                            argsBuf[32 + sizeof(ULONG)];
	 /*DWORD				bufSize ;
	 DWORD				dataSize ;
	 DWORD				dwArg ;*/
	 INT				threadID ;
	 /*SHORT				* pData ;

	 DSP_HSTREAM		hStream1;*/
	 BYTE				**arrBuf1;
	 /*SHORT *			sendBuffer ;
	 SHORT *			receiveBuffer ;*/
	 CHAR				* dllName;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = 0 ;
	pszUuid = targs->uuidPtr;
	dllName = targs->dllName;

	sem_post(&semCreate);


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	if (targs->transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }
    
    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    
   cbData = (struct DSP_CBDATA *)argsBuf ;    
 
   cbData->cbData = 32;
	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);


	// pszUuid is passed from calling function
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }
    
    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (status != DSP_EDYNLOAD) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    if (status == DSP_EDYNLOAD)
	{
        return PASS;
        }
        else
        {
				return FAIL;
			}

}



/*
 *  ======== main ========
 *
 */
INT dynCircDepLibCheck_STRESS(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;
    
    BOOL           			bVerbose=FALSE;
    FILE           			*pHandle = NULL; 

    INT            			iNumTransfers = DEFNUMTXFRS;
    INT            			iBuffSize = DEFBUFSIZE;
    
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	dynLinkArgs 			funcArgs;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
    INT                 	retVal = PASS;
	INT 					transferMode = STRMMODE_PROCCOPY;
	CHAR					*uuidPtr = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1"; // inout
	CHAR					*depuuidPtrb = "3B8934AD_E227_4531_B548_A9A69A239CB5"; // deplib	
	CHAR					*depuuidPtrc = "CD929A23_0EF8_4589_B496_C0479EE4087E";
	CHAR        			*aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	CHAR					*depDllPtrb = "/release/inoutcircdeplib2.dll55l";
	CHAR					*depDllPtrc = "/release/inoutcircdeplib3.dll55l";
	struct DSP_UUID				gidNodeID;
	CHAR					*depCreaDll = "/release/inoutCreatecircdepStress.dll55l";
	CHAR					*depExecDll = "/release/inoutExecutecircdepStress.dll55l";
	CHAR					*depDelDll = "/release/inoutDeletecircdepStress.dll55l";
	BOOL					fLoadDsp = FALSE;



	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Call the DepLibCheck func and pass the
	   params to this function */
	/* In the Function do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set Properties, dynamically Allocate,Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Call a function which is in the dep library */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
			break;
		case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;	
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
    status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
    }
    }
    
    /* Register the node information with DCD */
    UUID_UuidFromString((CHAR *)depuuidPtrb, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrb);
    if (!DSP_SUCCEEDED(status)) 
	{
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }

    UUID_UuidFromString((CHAR *)depuuidPtrc, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrc);
    
    UUID_UuidFromString((CHAR *)uuidPtr, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_NODETYPE, aDllName[0]);
    if (!DSP_SUCCEEDED(status)) 
	{
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }
    status = DSPManager_RegisterObject(&gidNodeID, DCD_CREATELIBTYPE, depCreaDll);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_EXECUTELIBTYPE, depExecDll);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_DELETELIBTYPE, depDelDll);

	/* Fill the funcArgs which are to be passed on to the Thread */
	funcArgs.hProcessor = hProcessor;
	funcArgs.iNumTransfers = iNumTransfers;
	funcArgs.iBuffSize = iBuffSize;
	funcArgs.pHandle = pHandle;
	funcArgs.bUseLog = bUseLog;
	funcArgs.bVerbose = bVerbose;
	funcArgs.dllName = aDllName[0];
	funcArgs.uuidPtr = uuidPtr;

	funcArgs.transferMode = transferMode;
	retVal = dynCircDepLibcheckFxn(&funcArgs);
		
    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

