/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MaxPerDepLib.c
 *
 *   Path    $(TESTSUITEROOT)/Stress/DynLink/
 * 
 *   Desc:
 *           This test case tests Dependant Library support of 
 *			 dynamic loading/linking Feature.
 *           The program stresses the number of Persistent Dependent
 *			 Libraries allowed. 
 *
 *  Usage:
 *        DYNMAXPERLIB [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -n: Number of Transfers for each Iteration
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 12, 2003    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <semaphore.h>
#include <DSPProcessor_OEM.h>
#include "dynLink.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define DEFNUMBUFS         1
#define DEFNUMTXFRS    10000
#define NUMOFCOMPARGS      2	/* Compulsory args: n,s */
#define PASS               0
#define FAIL               1

#define SHORT short

#define DSPEXEC         "inout_nortdxidl.dof"
#define DLLNAME         "/release/inoutdynstress.dll55l"
static sem_t semCreate = { {0,0}, 1 };


static DSP_STATUS SendTestConfiguration(SHORT **pOutBuf, 
										DSP_HSTREAM hOutStream, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize,
										UINT iNumTransfers);

/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Function for Dep Lib check test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents. Copy the contents to local variables    *
 * and use	*
 */
static INT maxPerDepLibcheckFxn(dynLinkArgs * targs)
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hStream;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	INT						i,k;
	DSP_STATUS				exitStatus = DSP_SOK;
	struct DSP_STREAMATTRIN        strmAttrs;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	 BYTE                            argsBuf[32 + sizeof(ULONG)];
	 DWORD				bufSize ;
	 DWORD				dataSize ;
	 DWORD				dwArg ;
	 INT				threadID ;
	 SHORT				* pData ;

	 DSP_HSTREAM		hStream1;
	 BYTE				**arrBuf1;
	 SHORT *			sendBuffer ;
	 SHORT *			receiveBuffer ;
	 CHAR				* dllName;

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	threadID = 0 ;
	pszUuid = targs->uuidPtr;
	dllName = targs->dllName;

	sem_post(&semCreate);


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = DEFNUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	if (targs->transferMode  == STRMMODE_PROCCOPY)
    {
		attrs.uSegid = 0;
    }
    else
    {
		attrs.uSegid = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }
    
    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

    
   cbData = (struct DSP_CBDATA *)argsBuf ;    
 
   cbData->cbData = 32;
	arrBuf = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);
	arrBuf1 = OsSpecificMemAlloc(sizeof(BYTE *) * DEFNUMBUFS);


	// pszUuid is passed from calling function
	///printf("UUID is = %s\n",pszUuid);
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        return FAIL;
    }
    
   //// printf("After Allocate\n");

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        return FAIL;
    }

    ////printf("After Connect\n");

    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        return FAIL;
    }
    
    ////printf("After Run\n");


	strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrs.uTimeout = 5000;
	strmAttrs.uAlignment = 0;
	strmAttrs.uNumBufs = DEFNUMBUFS;   /* To have 4 buffers in the Queue, for this test case */
	strmAttrs.lMode = targs->transferMode ;
	strmAttrs.uDMAChnlId = 0;
	strmAttrs.uDMAPriority = 0;

	if (strmAttrs.lMode  == STRMMODE_PROCCOPY)
        {
        	strmAttrs.uSegment = 0;
        }
        else
        {
	    strmAttrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
        }


    status = DSPStream_Open(hNode, DSP_TONODE, 0, &strmAttrs, &hStream);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream, iBuffSize, arrBuf, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, &strmAttrs, &hStream1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

    status = DSPStream_AllocateBuffers(hStream1, iBuffSize, arrBuf1, DEFNUMBUFS);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

    ////printf("After Strm open and alloc buf\n");
    
    status = SendTestConfiguration((SHORT **)arrBuf, hStream, 
    			iBuffSize, iBuffSize, (UINT)iNumTransfers);
	
        pData = (SHORT *)arrBuf[0] ;
        receiveBuffer = (SHORT *)arrBuf1[0] ;

        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */
        sendBuffer = OsSpecificMemAlloc(iBuffSize) ;

        for (k = 0 ; k < iNumTransfers ; k++)
	    {
		    pData[0]  = (BYTE)iBuffSize;
		    pData[1]  = (BYTE)0;
		    
		    ////printf("Buff size = %d\n",iBuffSize);
		    
		    for(i=0;i<iBuffSize/2;i++)
            {
	            pData[i]  = sendBuffer[i] = i + k ;
	        }

 			/* Issue Full Data Buffer */
			status = DSPStream_Issue(hStream, (BYTE *)pData, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Empty Buffer */
			status = DSPStream_Reclaim(hStream, (BYTE **)&pData, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}
			
			////printf("After IssuueRecl on Output Buffer\n");

			/* Issue Empty Buffer */
			status = DSPStream_Issue(hStream1, (BYTE *)receiveBuffer, iBuffSize, iBuffSize, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}

			/* Reclaim Full Buffer */
			status = DSPStream_Reclaim(hStream1, (BYTE **)&receiveBuffer, &dataSize, &bufSize,&dwArg);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}
			
			////printf("After IssuueRecl on Input Buffer\n");
		    
			/* Check for validity of Data */
		    /*for( i = 2 ;i< iBuffSize/2 ;i++)
		    {
			    if(sendBuffer[i] != receiveBuffer[i])
			    {
				    status = DSP_EFAIL ;
		        	    ////printf("Data Verification Failed, i=%d: Expected 0x%x Received 0x%x\n", i, sendBuffer[i],receiveBuffer[i]) ;
			    }
		    }*/

		    if ((k % 500) == 0)
		    {
			    ////printf("Thread ID:%d: Issue Reclaim:%d\n",threadID,k) ;
		    }
			    


	    } // End of For loop - iNumTransfers

	/* Free the Allocated Buffers */
	status = DSPStream_FreeBuffers(hStream, arrBuf, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream\n");
		}
		////printf("Error in DSPStraem_FreeBuffers()\n");
		return FAIL;
	}

	status = DSPStream_FreeBuffers(hStream1, arrBuf1, DEFNUMBUFS);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_FreeBuffers() for hStream1\n");
		}
		////printf("Error in DSPStraem_FreeBuffers() for hStream1\n");
		return FAIL;
	}

    status = DSPStream_Close(hStream);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream\n");
		}
		////printf("Error in DSPStraem_Close() for hStream\n");
		return FAIL;
	}

	status = DSPStream_Close(hStream1);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPStraem_Close() for hStream1\n");
		}
		////printf("Error in DSPStraem_Close() for hStream1\n");
		return FAIL;
	}

    status = DSPNode_Terminate(hNode, &exitStatus);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Terminate()\n");
		}
		////printf("Error in DSPNode_Terminate\n");
		return FAIL;
	}

    status = DSPNode_Delete(hNode);
	if (!DSP_SUCCEEDED(status)) 
	{
		/* Report Error */
		TST_verbose("FAIL\n");

		if (bUseLog) 
		{
			TSTLOG_appendLog(pHandle, "FAIL\n");
			TSTLOG_appendLog(pHandle,"Error in DSPNode_Delete()\n");
		}
		////printf("Error in DSPNode_Delete()\n");
		return FAIL;
	}

	OsSpecificMemFree(sendBuffer) ;
    /* Return from this Thread */
    if (DSP_SUCCEEDED(status))
    {
        return PASS;
    }
    else
    {
        return FAIL;
    }
}



/*
 *  ======== main ========
 *
 */
INT maxPerDepLibCheck_STRESS(INT argc, CHAR * argv[])
{
    BOOL           			bUseLog=FALSE;
    
    BOOL           			bVerbose=FALSE;
    FILE           			*pHandle = NULL; 

    INT            			iNumTransfers = DEFNUMTXFRS;
    INT            			iBuffSize = DEFBUFSIZE;
    
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	dynLinkArgs 			funcArgs;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
    INT                 	retVal = PASS;
	INT 					transferMode = STRMMODE_PROCCOPY;
	CHAR					*uuidPtr = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1"; // inout
	CHAR					*depuuidPtrb = "3B8934AD_E227_4531_B548_A9A69A239CB5"; // deplib	
	CHAR					*depuuidPtrc = "CD929A23_0EF8_4589_B496_C0479EE4087E";
	CHAR					*depuuidPtrd = "A755BB63_F0DE_4B09_B69B_4EF44C14D906";
	CHAR					*depuuidPtre = "9D3D419A_DE9F_4FE3_BE9E_48E245F650F0";
	CHAR					*depuuidPtrf = "BB8FB8D8_E0F4_4DA1_B1E7_64FD14EDEB2B";
	CHAR					*depuuidPtrg = "1CE452B0_2F9C_4721_853A_AC1BCAE74211";
	CHAR        			*aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
	CHAR					*depDllPtrb = "/release/inoutperdeplib21.dll55l";
	CHAR					*depDllPtrc = "/release/inoutperdeplib22.dll55l";
	CHAR					*depDllPtrd = "/release/inoutperdeplib23.dll55l";
	CHAR					*depDllPtre = "/release/inoutperdeplib24.dll55l";
	CHAR					*depDllPtrf = "/release/inoutperdeplib25.dll55l";
	CHAR					*depDllPtrg = "/release/inoutperdeplib26.dll55l";
	struct DSP_UUID				gidNodeID;
	CHAR					*depCreaDll = "/release/inoutCreatedynStress.dll55l";
	CHAR					*depExecDll = "/release/inoutExecutedynStress.dll55l";
	CHAR					*depDelDll = "/release/inoutDeletedynStress.dll55l";



	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Call the DepLibCheck func and pass the
	   params to this function */
	/* In the Function do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set Properties, dynamically Allocate,Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Call a function which is in the dep library */
		/* Do the Data Transfer */
	/* End !! */


	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DynamicLink_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "dynLink test:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
		case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;	
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, argv, "+vl:x:n:s:d:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
    status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
    }
    
    ////printf("After Proc Start\n");
    
    /* Register the node information with DCD */
    UUID_UuidFromString((CHAR *)depuuidPtrb, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrb);
    if (!DSP_SUCCEEDED(status)) 
	{
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }

    UUID_UuidFromString((CHAR *)depuuidPtrc, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrc);
    
    UUID_UuidFromString((CHAR *)depuuidPtrd, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrd);

    UUID_UuidFromString((CHAR *)depuuidPtre, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtre);

    UUID_UuidFromString((CHAR *)depuuidPtrf, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrf);

    UUID_UuidFromString((CHAR *)depuuidPtrg, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_LIBRARYTYPE, depDllPtrg);

    UUID_UuidFromString((CHAR *)uuidPtr, &gidNodeID);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_NODETYPE, aDllName[0]);
    if (!DSP_SUCCEEDED(status)) 
	{
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL status= 0x%x\n",status);
            TSTLOG_appendLog(pHandle,"Error in DSPManager_RegisterObject().\n");
        }
        return FAIL;
    }
    status = DSPManager_RegisterObject(&gidNodeID, DCD_CREATELIBTYPE, depCreaDll);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_EXECUTELIBTYPE, depExecDll);
    status = DSPManager_RegisterObject(&gidNodeID, DCD_DELETELIBTYPE, depDelDll);

	/* Fill the funcArgs which are to be passed on to the Thread */
	funcArgs.hProcessor = hProcessor;
	funcArgs.iNumTransfers = iNumTransfers;
	funcArgs.iBuffSize = iBuffSize;
	funcArgs.pHandle = pHandle;
	funcArgs.bUseLog = bUseLog;
	funcArgs.bVerbose = bVerbose;
	funcArgs.dllName = aDllName[0];
	funcArgs.uuidPtr = uuidPtr;

	funcArgs.transferMode = transferMode;
	retVal = maxPerDepLibcheckFxn(&funcArgs);
		
    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS) 
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

static DSP_STATUS SendTestConfiguration(SHORT **pOutBuf, 
										DSP_HSTREAM hOutStream, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize,
										UINT iNumTransfers)
{

		DSP_STATUS status = DSP_SOK;
		UINT			k;
		DWORD			dwArg;
		(*pOutBuf)[0] = dwSendBufSize / 2;
		(*pOutBuf)[1] = dwRecvBufSize / 2;
		(*pOutBuf)[2] = iNumTransfers & 0xffff ;
		(*pOutBuf)[3] = iNumTransfers >> 16 ;

		for ( k = 4; k < dwSendBufSize / 2; k++)
		{
				(*pOutBuf)[k] = k;
		}

		///printf("SendTestConfig: sending config\n");
		status = DSPStream_Issue(hOutStream, (BYTE *)*pOutBuf, 
        dwSendBufSize, dwSendBufSize, 0);
        if (DSP_FAILED(status)) 
		{
            ///printf("DSPStream_Issue %d failed.\n", k);
        }
        ///printf("SendTestConfig: after issue\n");

		status = DSPStream_Reclaim(hOutStream, 
						(BYTE **)pOutBuf, &dwRecvBufSize, NULL, &dwArg);
        if (DSP_FAILED(status)) 
		{
            /////printf("DSPStream_Reclaim %d failed.\n", k);
        }
		return status;
}
