/*  ============================================================================
*   File    IvaOvly.c
*
*   Path    $(TESTSUITEROOT)\Analysis\ivaovly
*
*   Rev     1.0.0
*
*       Desc    Tests the Overlay Nature of the IVA.
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2004
*
*   ============================================================================
*   Revision History
*
*   Nov 23, 2004  REF=IVA Overlay sample        Ramesh Gupta G
*
*   ============================================================================
*/





#include <stdlib.h>
#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>



/* Default stream attributes */
#define DEFAULTALIGNMENT   0
#define DEFAULTSEGMENT     0

#define DEFAULTIVAEXEC "./iva.dof"

#define ARGSIZE         32
#define DSPWORDSIZE     2           /* 1 DSP word = 2 bytes. */

#define OVLY1PF_CHANNEL 	( 11)
#define OVLY2PF_CHANNEL 	( 12)

/**** Host to PF messages**********/

#define OVLYNODE_CREATE 			( 1)
#define OVLYNODE_EXECUTE 			( 2)
#define OVLYNODE_DELETE 			( 3)
#define OVLYNODE_PING              	( 4)
#define OVLYNODE_PINGREPLY  		( 5)



/*
 *  ======== IvaOvly Function ========
 */

int IvaOvly(int argc, char ** argv)
{
    IVAMSG_TASK     IvaOvlyTask;
    DSP_STATUS      status = DSP_SOK;
    DspManager_Open(argc, NULL);
    int 	    opt = 0;


      optind = 0;
        while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:ZPR:")) != EOF) {
        switch (opt) {
       		 case 'v':
            /* verbose */
	            	TST_setVerbose(TRUE);
        		break;

             	 case 'l':
            /* use log and store with name location */
                        strncpy(szLogName, optarg, sizeof(szLogName));
                        pHandle = TSTLOG_openLog(szLogName);
                        fOpenLog = TRUE;
                        TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
                        break;

	        case 'x':
            /* use given executable */
                        aArgv[0] = (char *) optarg;
                        fLoadDsp = TRUE;
                        TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
                        break;

                case 'n':
            /* set Number of transfers */
                        IvaOvlyTask.numMsg = CSL_Atoi(optarg);
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Number of Data Transfers =  %i\n", IvaOvlyTask.numMsg);
                        break;

                case 'D':
                        /* set Transfer Mode as DMMCOPY */
                /*      NodeArgs.iMode = STRMMODE_ZEROCOPY;
                        NodeArgs.iSegmentId = DSP_SHMSEG0;
                        if(fOpenLog)
                                TSTLOG_appendLog(pHandle, "Transfer Mode Set to =  %i\n", STRMMODE_ZEROCOPY);
                */
                        break;

		default:
	            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
        		break;
        }
    }




    /* Process command line arguments, open data files: */
	IvaOvlyTask.hProcessorIva = NULL;
	IvaOvlyTask.hNodeIva = NULL;

    if (DSP_SUCCEEDED(status)) {
        /* Perform processor level initialization. */
        status = InitializeIvaOvlyProcessor(&IvaOvlyTask);
    }

    if (DSP_SUCCEEDED(status)) 
	{
        /* Perform node level initialization. */
        status = InitializeIvaOvlyNode(&IvaOvlyTask, (struct DSP_UUID*)&IVA_OVLY1_TI_uuid);
         if (DSP_SUCCEEDED(status)) 
	   {
                /* Run task. */
                status = RunIvaOvlyTask(&IvaOvlyTask);
                if (DSP_SUCCEEDED(status)) 
		{
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY1 node succeeded.\n");
    				CleanupIvaOvlyNode(&IvaOvlyTask);
                }
                else 
		{
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY1 node failed.\n");
                }
            }
        }
    
	if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
        status = InitializeIvaOvlyNode(&IvaOvlyTask, (struct DSP_UUID*)&IVA_OVLY2_TI_uuid);
            if (DSP_SUCCEEDED(status)) {

                /* Run task. */
                status = RunIvaOvlyTask(&IvaOvlyTask);
                if (DSP_SUCCEEDED(status)) {
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY2 node succeeded.\n");
			CleanupIvaOvlyNode(&IvaOvlyTask);
                }
                else {
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY2 node failed.\n");
                }
            }
        }

	if (DSP_SUCCEEDED(status))
        {
        /* Perform node level initialization. */
        status = InitializeIvaOvlyNode(&IvaOvlyTask, (struct DSP_UUID*)&IVA_OVLY1_TI_uuid);
         if (DSP_SUCCEEDED(status))
           {
                /* Run task. */
                status = RunIvaOvlyTask(&IvaOvlyTask);
                if (DSP_SUCCEEDED(status))
                {
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY1 node 2nd time  succeeded.\n");
                }
                else
                {
                    TSTLOG_appendLog(pHandle,"RunTask of OVLY1 node 2nd time failed.\n");
                }
            }
        }



    CleanupIvaOvlyNode(&IvaOvlyTask);
    CleanupIvaOvlyProcessor(&IvaOvlyTask);

    DspManager_Close(0,NULL);

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeIvaOvlyProcessor(IVAMSG_TASK *IvaOvlyTask)
{  
        DSP_STATUS              status = DSP_EFAIL;
	INT		        argc    = 1 ;
	const CHAR *	        argvIva = aArgv;
	struct DSP_PROCESSORINFO	dspInfo;
	UINT			numProcs;
	UINT			index=0;
	INT			procId;

    /* Attach to IVA */
	while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index, &dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
	{
		if (dspInfo.uProcessorType == IVA_ARM7)
		{
			procId = index;
			status = DSP_SOK;	
			break;	
		}
		index++;	
	}

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&IvaOvlyTask->hProcessorIva);
        if (DSP_SUCCEEDED(status)) {
          TSTLOG_appendLog(pHandle,"DSPProcessor_Attach succeeded.:IVA\n");
        }
        else {
          TSTLOG_appendLog(pHandle,"DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
	else
         TSTLOG_appendLog(pHandle,"Failed to get the desired processor \n");

    if (DSP_SUCCEEDED(status))
    {
		TSTLOG_appendLog(pHandle,"IVA Image: %s. \n",aArgv);

	   status = DSPProcessor_Load(IvaOvlyTask->hProcessorIva,argc,aArgv,aEnvp);

	   if (DSP_SUCCEEDED(status))
	   {
		   TSTLOG_appendLog(pHandle,"DSPProcessor_Load:IVA Succeeded") ;
	   }
	   else
	   {
		   TSTLOG_appendLog(pHandle,"DSPProcessor_Load:IVA Failed.\n") ;
	   }
	   status = DSPProcessor_Start(IvaOvlyTask->hProcessorIva) ;

	   if (DSP_SUCCEEDED(status))
	   {
		   TSTLOG_appendLog(pHandle,"DSPProcessor_Start:IVA succeeded.\n") ;
	   }
	   else
	   {
		   TSTLOG_appendLog(pHandle,"DSPProcessor_Start:IVA failed.\n") ;
	   }

    }
    return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeIvaOvlyNode(IVAMSG_TASK *IvaOvlyTask, struct DSP_UUID *pUuid)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgsDsp;
    struct DSP_NODEATTRIN  nodeAttrInIva;
    DSP_STATUS      status = DSP_SOK;

    nodeAttrInIva.uTimeout = DSP_FOREVER;
    nodeAttrInIva.iPriority = 5;

    pArgsDsp = (struct DSP_CBDATA *)argsBuf;
    pArgsDsp->cbData = ARGSIZE;

	/* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(IvaOvlyTask->hProcessorIva, pUuid, pArgsDsp,&nodeAttrInIva, &IvaOvlyTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,"DSPNode_Allocate succeeded.:IVA \n");
        }
        else {
            TSTLOG_appendLog(pHandle,"DSPNode_Allocate failed.:IVA Status = 0x%x\n",status);
        }
    }

	if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(IvaOvlyTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,"DSPNode_Create succeeded.:IVA\n");
        }
        else {
           TSTLOG_appendLog(pHandle,"DSPNode_Create failed.:IVA Status = 0x%x\n", status);
        }
    }

	    /* Start the node on the IVA. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(IvaOvlyTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,"DSPNode_Run succeeded.:IVA \n");
        }
        else {
            TSTLOG_appendLog(pHandle,"DSPNode_Run failed.:IVA Status = 0x%x\n",status);
        }
    }

    return (status);
}

/*
 *  ======== RunTask ========
 *  Run ivamsg task.
 */
static DSP_STATUS RunIvaOvlyTask(IVAMSG_TASK *IvaOvlyTask)
{
        DSP_STATUS      status = DSP_SOK;
	struct DSP_MSG		message;
	INT		n;
	DWORD           dwStartTime;
        DWORD           dwElapsedTime;
        DWORD           dwEndTime;
	ULONG           ulNumBytesTransferred = IvaOvlyTask->numMsg;
	ULONG           ulBytesPerSec;

	OsSpecificTimerStart(&dwStartTime);
	for (n=0;n<IvaOvlyTask->numMsg;n++)
	{
				
		message.dwCmd = OVLYNODE_PING;
		status = DSPNode_PutMessage(IvaOvlyTask->hNodeIva, &message, 15000);

    		if (!DSP_SUCCEEDED(status)) 
		{
        	TSTLOG_appendLog(pHandle,"DSPNode_PutMessage:IVA failed, 0x%x.\n", status);
     		}

		status = DSPNode_GetMessage(IvaOvlyTask->hNodeIva, &message, 15000);
				
		if (!DSP_SUCCEEDED(status))
		{
        	TSTLOG_appendLog(pHandle,"DSPNode_GetMessage:IVA failed, 0x%x.\n",status);
     		}

		if (message.dwCmd != OVLYNODE_PINGREPLY)
		{
          	TSTLOG_appendLog(pHandle,"DSPNode_GetMessage:IVA Message Integrity failed, 0x%x.\n",message.dwCmd);
			break;
		}


	
			
	}

	 OsSpecificTimerStop(&dwEndTime);
         OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);

	if (DSP_SUCCEEDED(status))
                {
                    ulNumBytesTransferred = IvaOvlyTask->numMsg;
                    ulBytesPerSec = (ulNumBytesTransferred * 1000) / dwElapsedTime;
                    if(fOpenLog)
                    {
                            TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
                            TSTLOG_appendLog(pHandle, "LOG: *********IVAOVLY STATISTICS**********\n");
                            TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
                            TSTLOG_appendLog(pHandle, "LOG: BufferSize = Not Applicable\n");
                            TSTLOG_appendLog(pHandle, "LOG: StartTime = %d\n", dwStartTime);
                            TSTLOG_appendLog(pHandle, "LOG: EndTime = %d\n", dwEndTime);
                            TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %d\n", dwElapsedTime);
                            TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Messages/sec\n", ulBytesPerSec);
                            TSTLOG_appendLog(pHandle, "LOG: Number of Transfers = %d\n", ulNumBytesTransferred);
                            TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
                            TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");


	            }
        	}
	else
		{
			 TSTLOG_appendLog(pHandle,"Error FINAL STATUS is %x \n",status);
		}


    return (status);
}

/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupIvaOvlyNode(IVAMSG_TASK *IvaOvlyTask)
{
    DSP_STATUS      status;
    DSP_STATUS      exitStatus;

    if (IvaOvlyTask->hNodeIva) {

		// Terminate IVA node.
        status = DSPNode_Terminate(IvaOvlyTask->hNodeIva, &exitStatus);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,"DSPNode_Terminate:IVA succeeded.\n");
        }
        else {
           TSTLOG_appendLog(pHandle,"DSPNode_Terminate:IVA failed: 0x%x\n", status);
        }

		/* Delete IVA node. */
        status = DSPNode_Delete(IvaOvlyTask->hNodeIva);
        if (DSP_SUCCEEDED(status)) {
           TSTLOG_appendLog(pHandle,"DSPNode_Delete:IVA succeeded.\n");
        }
        else {
           TSTLOG_appendLog(pHandle,"DSPNode_Delete:IVA failed: 0x%x\n", status);
        }

       	IvaOvlyTask->hNodeIva = NULL;

    }
	else
	{
        	TSTLOG_appendLog(pHandle,"Invalid Node Handle \n");
	}

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS CleanupIvaOvlyProcessor(IVAMSG_TASK *IvaOvlyTask)
{
    DSP_STATUS      status;

	if (IvaOvlyTask->hProcessorIva) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(IvaOvlyTask->hProcessorIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach succeeded:IVA.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed:IVA.\n");
        }

        IvaOvlyTask->hProcessorIva = NULL;
    }


    return (status);
}



