/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    stress.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\stress\stress.c
 * 
 *   Desc:
 *        Automated Environment for running GPP side Stress Tests 
 *        "stress -v" will run all tests verbose.
 *        "stress -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *  Usage:
 *        stress [-v] [-l "log path/filename"] 
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *        -v: verbose mode
 *        -l: use log and location to store results
 *        -x: DSP executable to be used in testing
 *        -m: individual module to test or ALL for all modules
 *            (Default = ALL)
 *        -n: list of API functions (see below) or ALL for all functions
 *            (Default = ALL)
 *        -#: number of times to run each API function test
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 31, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <os.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <testcsl.h>

INT delayTransfer_STRESS (INT argc, CHAR * argv[] );
INT lowResource_STRESS (INT argc, CHAR * argv[] );
#ifndef __KERNEL__
INT multiProcessData_STRESS (INT argc, CHAR * argv[] );
INT multiThreadData_STRESS (INT argc, CHAR * argv[] );
INT multiThreadMsg_STRESS (INT argc, CHAR * argv[] );
INT analysisMultiThreadData_STRESS(INT argc, CHAR * argv[] );
INT multiThreadDataQueue_STRESS(INT argc, CHAR * argv[] );
INT overlay_test0_STRESS(INT argc, CHAR * argv[] );
INT overlay_test1_STRESS(INT argc, CHAR * argv[] );
INT multiThreadMsgQueue_STRESS(INT argc, CHAR * argv[] );
INT multiThreadPrioDataQueue_STRESS(INT argc, CHAR * argv[]);
INT multiThreadPrioDMM_STRESS(INT argc, CHAR * argv[]); 
#endif

/* 
 *  ======== stress_main ======== 
 */
INT stress_main (INT argc, CHAR * argv[])
{

    CHAR        * tmpSuiteName = "ALL";
    CHAR        ** tmp_argv;
    INT         tmp_argc;
	
    tmpSuiteName = argv[1];


    /* Now drop the first parameter and pass the rest of the
    parameters to corresponding Test Suite */
    tmp_argc = argc;  tmp_argc--;
    argv++;  tmp_argv = argv;


    if (!CSL_Strcmp(tmpSuiteName, "DELAY"))
    {
        delayTransfer_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "LOWRES"))
    {
        lowResource_STRESS(tmp_argc, tmp_argv);
    }
#ifndef __KERNEL__
    else if (!CSL_Strcmp(tmpSuiteName, "MULPROC"))
    {
        multiProcessData_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDDATA"))
    {
        multiThreadData_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDMSG"))
    {
        multiThreadMsg_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "AMULTHRDDATA"))
    {
        analysisMultiThreadData_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDDATAQ"))
    {
        multiThreadDataQueue_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRPRIODDATAQ"))
    {
        multiThreadPrioDataQueue_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDMSGQ"))
    {
        multiThreadMsgQueue_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDDMM"))
    {
        multiThreadPrioDMM_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "OVLY0"))
    {
        overlay_test0_STRESS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "OVLY1"))
    {
	 overlay_test1_STRESS(tmp_argc, tmp_argv);

    }
#endif
    else
    {
        TST_verbose("Invalid test suite specified: %s\n", tmpSuiteName);
    }

    return 0;
}


