/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPManager_UnregisterObject.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPManager\
 *
 *   Desc:
 *        DSPManager API test
 *
 *  DSPManager API function tested:
 *        DSPManager_UnregisterObject(struct DSP_UUID *pUuid,
 *						DSP_DCDOBJTYPE objType, CHAR *pszPathName);
 *  Usage:
 *        [-v] [-l "log path/filename"] [-x "valid executable"]
 *        [-d "valid dll for node"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUN 17, 2003    REF=ORG             Swamy MC
 *      Created
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
//#include <dbdcddef.h>

#define DEFNAMESIZE    80

#define DSPEXEC         "dataxfr_nortdx_int_24xx.dof55L"
#define DLLNAME         "dataxfrdll_24xx.dll55L"

/*
 *  Function to test DSPManager_UnregisterObject API
 */
INT DSPManager_UnregisterObject_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i, j;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR        *aDllName[DEFNAMESIZE] = { DLLNAME, "arg3" };
    CHAR        *pszUuid;
    DSP_STATUS        status;


    /* INPUT PARAMETERS */
    struct DSP_UUID *arg1[] =     /* arg 1: pointer to struct DSP_UUID */
    {
        NULL, /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    DSP_DCDOBJTYPE arg2[] =      /* arg 2:  */
    {
        DSP_DCDNODETYPE,       /* valid object type */
        DSP_DCDLIBRARYTYPE,
	    DSP_DCDCREATELIBTYPE,
	    DSP_DCDEXECUTELIBTYPE,
	    DSP_DCDDELETELIBTYPE,
        10                  /* invalid value for Obj type */
    };
        					/* DSP_DCDLIBRARYTYPE,      valid */

    #define NUMARG1 (sizeof(arg1)/sizeof(struct DSP_UUID *))
    #define NUMARG2 (sizeof(arg2)/sizeof(DSP_DCDOBJTYPE))

    /* SOLUTION SET */
    struct
    {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] = /* arg1 arg2 */

    {
        {
            {DSP_SOK,   TST_SHOULDEQUAL}, /* [0][0] */
            {DSP_SOK,   TST_SHOULDEQUAL}, /* [0][1] */
            {DSP_SOK,   TST_SHOULDEQUAL}, /* [0][2] */
            {DSP_SOK,   TST_SHOULDEQUAL}, /* [0][3] */
            {DSP_SOK,   TST_SHOULDEQUAL}, /* [0][4] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}  /* [0][5] */
        },
        {
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][0] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][1] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][2] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][3] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][4] */
            {DSP_EINVALIDARG, TST_SHOULDEQUAL}  /* [0][5] */
        }
    };


    /* Initialise pHandle */
    pHandle = (FILE *)NULL;

    arg1[0] = OsSpecificMemAlloc(sizeof(struct DSP_UUID));

	pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1"; // dataxfr
    UUID_UuidFromString((CHAR *)pszUuid, arg1[0]);

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF)
    {
        switch (opt)
        {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPManager_UnregisterObject: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPManager_UnregisterObject:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
    	case 'x':
        	/* use given executable */
        	aArgv[0] = (char *) optarg;
            break;
    	case 'd':
        	/* use given dll */
        	aDllName[0] = (char *) optarg;
            break;
        default:
            TST_reportError("Invalid option: %c\n", opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSP dynamic DLL: %s\n", aDllName[0]);

    while (cRetVal > 0)
    {
        for (i = 0; i < NUMARG1; i++)
        {
            for (j = 0; j < NUMARG2; j++)
            {
	            if((i == 0) && (j != 5))
	            {
		            status = DSPManager_RegisterObject(arg1[i], arg2[j],aDllName[0]);
	            }
                status = DSPManager_UnregisterObject(arg1[i], arg2[j]);

                if (!TST_CheckVal(status,
                    checkArray[i][j].dwValue,
                    checkArray[i][j].fEqualFlag))
                {
                    cRetVal = 0;
                    TST_verbose("Reg Obj [%d][%d] "
                    	"returned %x\n", i, j, status);

                    if (bUseLog)
                    {
                        TSTLOG_appendLog(pHandle, "FAIL : status = 0x%x\n",status);
                        TSTLOG_appendLog(pHandle, "Unregister Object [%d][%d] "
                        "failed.\n", i, j);
                    }
                }
            }
        }

        break;
    }

    OsSpecificMemFree(arg1[0]);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

