/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPNode_Allocate.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPNode\DSPNode_Allocate.c
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_Allocate(DSP_HPROCESSOR hProcessor, struct DSP_UUID pNodeID,
 *                         DSP_CBDATA *pArgs, DSP_NODEATTRIN *pAttrIn,
 *                         DSP_HNODE *phNode)
 *  Usage:
 *        DSPNode_Allocate [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC         "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPNode_Allocate_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i, j, k, l, m;
    INT            iArgc = 1;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR        *pszUuid;
    DSP_STATUS        status;
    DSP_STATUS tmpStatus;
    struct DSP_PROCESSORATTRIN    *pAttrIn;
    DSP_STATUS exitStatus = DSP_SOK;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =     /* arg 1: attached processor handle */
    {
        NULL, /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    struct DSP_UUID *arg2[] =      /* arg 2: pointer to struct DSP_UUID */
    {
        NULL, /* valid pointer to be allocated */
        NULL                /* invalid NULL pointer */
    };

    struct DSP_CBDATA *arg3[] =    /* arg 3: pointer to CBDATA structure */
    {
        NULL, /* valid pointer to be allocated */
        NULL              /* invalid NULL pointer */
    };

    struct DSP_NODEATTRIN *arg4[] =  /* arg 4: pointer to NODEATTRIN structure */
    {
        NULL, /* valid pointer to be allocated */
        NULL              /* invalid NULL pointer */
    };

    /*
     * Use pointers to the handle so that we can test a NULL pointer
     * as well as a valid one that points to a valid handle
     */
    DSP_HNODE *arg5[] =   /* arg 5: pointer location for node handle */
    {
        NULL, /* valid location to be allocated */
        NULL                /* NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_UUID *))
    #define NUMARG3 (sizeof(arg3)/sizeof(struct DSP_CBDATA *))
    #define NUMARG4 (sizeof(arg4)/sizeof(struct DSP_NODEATTRIN *))
    #define NUMARG5 (sizeof(arg5)/sizeof(DSP_HNODE))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
                                                /* arg1 arg2 arg3 arg4 arg5 */
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */
                    },
                    {
                        {DSP_ERANGE, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                    },

                },
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                    },
                    {
                        {DSP_ERANGE, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                    },
                },
            },
        },
    };


    /* Initialise pHandle */
    pHandle = (FILE *)NULL;
    
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_UUID));

	pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1"; // dataxfr
    UUID_UuidFromString((CHAR *)pszUuid, arg2[0]);

    arg3[0] = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    arg3[0]->cbData=1;

    arg4[0] = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    arg4[0]-> cbStruct = sizeof(struct DSP_NODEATTRIN);
    arg4[0]->uTimeout = 1000;
    arg4[0]->iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    arg4[0]->uProfileID = 0xff;
#endif


    arg5[0] = OsSpecificMemAlloc(sizeof(DSP_HNODE));

    /* iPriority out of range */
    arg4[1]  = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    arg4[1]-> cbStruct = sizeof(struct DSP_NODEATTRIN);
    arg4[1]->uTimeout = 1000;
    arg4[1]->iPriority = 30;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    arg4[1]->uProfileID = 0xff;
#endif

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_Allocate: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
    case 'x':
        /* use given executable */
        aArgv[0] = (char *) optarg;
        break;
        default:
            TST_reportError("Invalid option: %c\n", opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_Allocate: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;

        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
    
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()");
            }
            break;
        }
        /* Load executable */
        status = DSPProcessor_Load(arg1[0], iArgc, (CONST CHAR **)aArgv, NULL);
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load() and status = 0x%x\n",status);
            }
            break;
        }

        /* Start Processor */
        status = DSPProcessor_Start(arg1[0]);
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start()");
            }
            break;
        }


        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {
                        for (m = 0; m < NUMARG5; m++) {

                            status = DSPNode_Allocate(arg1[i], arg2[j],
                                             arg3[k], arg4[l], arg5[m]);
			     		tmpStatus = status;

				if(DSP_SUCCEEDED(status))
				{
					status = DSPNode_Terminate(*arg5[m],&exitStatus);
    					if(DSP_SUCCEEDED(status))
					{
            				  TSTLOG_appendLog(pHandle, "DSPNode_Terminate PASS and status = 0x%x\n",status);
					}
					status = DSPNode_Delete(*arg5[m]);
    					if(!DSP_SUCCEEDED(status))
					{
            				  TSTLOG_appendLog(pHandle, "DSPNode_Delete  FAIL and status = 0x%x\n",status);
					}
					  status = tmpStatus;
				}

                            if (!TST_CheckVal(status,
                                checkArray[i][j][k][l][m].dwValue,
                                checkArray[i][j][k][l][m].fEqualFlag)) {
                                cRetVal = 0;
                                TST_verbose(
                              "Allocate [%d][%d][%d][%d][%d] returned %x\n",
                                        i, j, k, l, m, status);

                                if (bUseLog) {
                                    TSTLOG_appendLog(pHandle, "FAIL\n");
                                    TSTLOG_appendLog(pHandle,
                                    "DSPNode_Allocate [%d][%d][%d][%d][%d] \
                                    failed.\n", i, j, k, l, m);
                                }
                            }
                        }
                    }
                }
            }
        }
        break;
    }

    status = DSPProcessor_Stop(arg1[0]);
    if(!DSP_SUCCEEDED(status))
	{
            TSTLOG_appendLog(pHandle, "DSPProcessor_Stop FAIL and status = 0x%x\n",status);
	}
    status = DSPProcessor_Detach(arg1[0]);
    if(!DSP_SUCCEEDED(status))
	{
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach FAIL and status = 0x%x\n",status);
	}

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arg3[0]);
    OsSpecificMemFree(arg4[0]);
    OsSpecificMemFree(arg4[1]);
    OsSpecificMemFree(arg5[0]);
	
    if (cRetVal&&(DSP_SUCCEEDED(status))) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

