/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPNode_RegisterNotify.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPNode\DSPNode_RegisterNotify.c
 * 
 *   Desc:
 *        DSPNode API test
 *
 *  DSPNode API function tested:
 *        DSPNode_RegisterNotify(DSP_HNODE hNode, UINT uEventMask,
 *                               UINT uNotifyMask, HANDLE hNotification)
 *  Usage:
 *        DSPNode_RegisterNotify [-v] [-l "log path/filename"]
 *                       [-x "valid executable"]
 *                       [-m "message to be sent"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: name of valid DSP executable
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>


#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT  0
#define DSPEXEC         "apitest1_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPNode_RegisterNotify_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            iArgc = 1;
    INT            i, j, k, l;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR        *pszUuid;
    DSP_HPROCESSOR    hProcessor;
    struct DSP_UUID        gidNodeID;
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_NODEATTRIN    *nodeAttrin;
    struct DSP_CBDATA        *cbData;


    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =      /* arg1: handle of node */
    {
        NULL, /* Valid handle to be allocated */
        NULL, /* Invalid NULL handle */
    };

    UINT arg2[] =           /* arg2: Event mask */
    {
        DSP_NODEMESSAGEREADY, /* valid mask */
        0x00000500                   /* invalid mask */
    };

    UINT arg3[] =           /* arg3: Notify Mask */
    {
        DSP_SIGNALEVENT, /* Valid mask */
        DSP_SIGNALEVENT +
        2             /* Invalid mask DSP_POSTMESSAGE  =2*/
    };

    struct DSP_NOTIFICATION* arg4[] =         /* arg4: handle used for notification */
    {
        NULL, /* valid handle to be allocated */
        NULL                         /* invalid NULL handle */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))
    #define NUMARG4 (sizeof(arg4)/sizeof(struct DSP_NOTIFICATION*))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][0][1] */
                },
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][0][1] */
                },
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][1] */
                },
            },
        },
        {
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1] */
                },
            },
            {
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1] */
               },
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1] */
               },
            },
        },
    };


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    nodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 1000;
    nodeAttrin->iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin->uProfileID = 0xff;
#endif



    cbData = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData = 0;

    arg4[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION*));

    pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:m:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_RegisterNotify: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode_RegisterNotify:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }

            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_RegisterNotify: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()\n");
            }

            break;
        }
        arg1[1] = arg1[0];
        /* Load Processor with Executable */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        /* Start Processor */
        status = DSPProcessor_Start(hProcessor);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load()\n");
            }

            break;
        }

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, nodeAttrin,
                   &arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Allocate()\n");
            }

            break;
        }

     /* Set Bogus Notification event */
        arg4[0]->psName = (PSTRING) "Event";

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {

                        status = DSPNode_RegisterNotify(arg1[i], arg2[j],
                                                        arg3[k], arg4[l]);

                        if (!TST_CheckVal(status,
                            checkArray[i][j][k][l].dwValue,
                            checkArray[i][j][k][l].fEqualFlag)) {
                            cRetVal = 0;
                            TST_verbose(
                            "RegisterNotify [%d][%d][%d][%d] returned %x\n",
                                           i, j, k, l, status);

                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                "DSPNode_Run [%d][%d][%d][%d] failed.\n",
                                     i, j, k, l);
                            }
                        }
                    }
                }
            }
        }

        status = DSPNode_Delete(arg1[0]);
        break;
    }

    status = DSPProcessor_Detach(hProcessor);

	OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(nodeAttrin);
    OsSpecificMemFree(cbData);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

