/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_EnumNodes.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_EnumNodes.c
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_EnumNodes(DSP_HPROCESSOR hProcessor,
 *                                  IN DSP_HNODE *aNodeTab,
 *                                  IN UINT uNodeTabSize,
 *                                  OUT UINT *puNumNodes,
 *                                  OUT UINT *puAllocated);
 *  Usage:
 *        DSPProcessor_EnumNodes [-v] [-l "log path/filename"] [-x "valid executable"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: name of valid DSP executable
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <uuidutil.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC        "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPProcessor_EnumNodes_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            iArgc = 1;
    INT            i, j, k, l, m;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    struct DSP_UUID        gidNodeID;
    DSP_STATUS        status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    DSP_HNODE		hNode = NULL;
    CHAR        	*pszUuid;
    struct DSP_STRMATTR	attrs;



    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =      /* arg1: handle of attached processor */
    {
        NULL, /* Valid handle to be attached */
        NULL, /* Invalid NULL handle */
    };

    DSP_HNODE *arg2[] =          /* first location of node handle array */
    {
        NULL, /* Valid pointer to handle to be allocated */
        NULL             /* Invalid NULL pointer */
    };

    UINT arg3[] =               /* arg2: number of handles in memory */
    {
        50, /* valid number */
        0                     /* invalid number */
    };

    UINT *arg4[] =               /* Pointer to number of valid nodes */
    {
        NULL, /* valid pointer to be allocated */
        NULL             /* invalid NULL pointer */
    };

    UINT *arg5[] =                 /* arg3:pointer to number of allicated nodes */
    {
        NULL, /* valid pointer */
        NULL                   /* invalid NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(DSP_HNODE *))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))
    #define NUMARG4 (sizeof(arg4)/sizeof(UINT *))
    #define NUMARG5 (sizeof(arg5)/sizeof(UINT *))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
                                                /* arg1 arg2 arg3 arg4 arg5 */
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                    },

                },
                {
                    {
                        {DSP_ESIZE, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                    },
                    {
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                    },
                },
            },
        },
    };

    /* Allocate pointer for dummy attributes to use */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;
    
    arg2[0] = OsSpecificMemAlloc(sizeof(DSP_HNODE));

    /* Allocate argument pointer */
    arg4[0] = OsSpecificMemAlloc(sizeof(UINT));

	arg5[0] = OsSpecificMemAlloc(sizeof(UINT));

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
         switch (opt) {
         case 'v':
             /* verbose */
             TST_setVerbose(TRUE);
             TST_verbose("DSPProcessor_EnumNodes: Verbose Mode On\n");
             break;
         case 'l':
             /* use log and store with name location */
             strncpy(szLogName, optarg, sizeof(szLogName));

            bUseLog = TRUE;
             TST_verbose("Opening %s\n", szLogName);
             pHandle = TSTLOG_openLog(szLogName);

             TSTLOG_appendLog(pHandle, "DSPProcessor_EnumNodes:\n");
             /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
             TST_verbose("Log opened as: %s\n", szLogName);
                break;
         case 'x':
             /* use DSP executable */
             aArgv[0] = (char *) optarg;
             break;
         default:
             TST_verbose(("Invalid option: %c\n"), opt);
             cRetVal = 0;
             if (bUseLog) {
                 TSTLOG_appendLog(pHandle,
                 "Invalid option specified.\n");
             }
             break;
         }
    }

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPProcessor_EnumNodes: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;
           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPProcessor_Attach.\n");
           }
           break;
       }

       status = DSPProcessor_Load(arg1[0], iArgc, (CONST CHAR **)aArgv, NULL);

       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;
           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPProcessor_Load().\n");
           }
           break;
       }

       status = DSPProcessor_Start(arg1[0]);

       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;
           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPProcessor_Start().\n");
           }
           break;
       }

	   pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1"; // dataxfr
       UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

       /* Allocate Node */
       status = DSPNode_Allocate(arg1[0], &gidNodeID, NULL, NULL,
                                 &hNode);

       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;

           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPNode_Allocate()\n");
           }
           break;
       }

		/* The following two Connect calls are added to accomodate the
		   usage of dataxfr dof instead of the apitest0 dof. The DSP
		   side expects the Connect calls. So these are added here */
		attrs.uSegid = 0;
		attrs.uBufsize = 512;
		attrs.lMode = STRMMODE_PROCCOPY;
		attrs.uNumBufs = 1;
		attrs.uAlignment = 0;
		attrs.uTimeout = 5000;

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }


        /* Create Node on DSP */
        status = DSPNode_Create(hNode);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Create()\n");
            }
            break;
        }

        /* Start Node running on DSP */
        status = DSPNode_Run(hNode);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Run()\n");
            }
            break;
        }

       for (i = 0; i < NUMARG1; i++) {
           for (j = 0; j < NUMARG2; j++) {
               for (k = 0; k < NUMARG3; k++) {
                   for (l = 0; l < NUMARG4; l++) {
                       for (m = 0; m < NUMARG5; m++) {
                           status = DSPProcessor_EnumNodes(arg1[i],
                                    arg2[j], arg3[k], arg4[l], arg5[m]);

                           if (!TST_CheckVal(status,
                               checkArray[i][j][k][l][m].dwValue,
                               checkArray[i][j][k][l][m].fEqualFlag)) {
                               cRetVal = 0;
                               TST_verbose(
                                "EnumNodes [%d][%d][%d][%d][%d] returned \
                                  %x\n", i, j, k, l, m, status);

                               if (bUseLog) {
                                   TSTLOG_appendLog(pHandle, "FAIL\n");
                                   TSTLOG_appendLog(pHandle,
                                   "DSPProcessor_EnumNodes \
                                   [%d][%d][%d][%d][%d] failed.\n",
                                   i, j, k, l, m);
                               }
                           }
                       }
                   }
               }
           }
       }


       DSPNode_Terminate(hNode, &exitStatus);
       DSPNode_Delete(hNode);

       /* Detach processor */
       status = DSPProcessor_Detach(arg1[0]);

       break;
    }

    OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(arg2[0]);
	OsSpecificMemFree(arg4[0]);
	OsSpecificMemFree(arg5[0]);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

