/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPStream_Open.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPStream\DSPStream_Open.c
 * 
 *   Desc:
 *        DSPStream_Open API test
 *
 *  DSPStream API function tested:
 *        DSPStream_Open(DSP_HNODE hNode, UINT uDirection,
 *                       UINT uIndex, DSP_STREAMATTRIN *pAttrIn,
 *                       DSP_HSTREAM *phStream)
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC        "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPStream_Open_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT            opt;
    INT            cRetVal = 1;
    INT            i, j, k, l, m;
    INT            iArgc = 1;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                *pszUuid;
    DSP_HPROCESSOR     hProcessor;
    struct DSP_UUID        gidNodeID;
    DSP_STATUS        status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_CBDATA        *cbData;
    struct DSP_NODEATTRIN    nodeAttrin;
    struct DSP_STRMATTR    attrs;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    BYTE        argsBuf[32 + sizeof(ULONG)];


    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =     /* arg 1: Handle of the task node */
    {
        NULL, /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    UINT arg2[] =      /* arg 2: Stream Direction to struct DSP_UUID */
    {
        DSP_TONODE, /* valid direction */
        0xFFFFFFFF         /* invalid direction */
    };

    UINT arg3[] =      /* arg 3: Stream index */
    {
        0, /* valid index */
        50                      /* invalid index */
    };

    struct DSP_STREAMATTRIN arg4[] =  /* arg 4: pointer to strm attributes */
    {
        {
			(DWORD)sizeof(struct DSP_STREAMATTRIN), /* cbStruct size */
			(UINT)5000, /* timeout */
			(UINT)0, /* segid */
			(UINT)0, /* allignment */
			(UINT)1, /* numbufs */
			(DSP_STRMMODE) STRMMODE_PROCCOPY, /* strm mode */
			(UINT)0, /* uDMAChnlId */
			(UINT)0, /* uDMAPriority */
	    }, /* valid pointer using PROCCOPY */

        {
			(DWORD)sizeof(struct DSP_STREAMATTRIN), /* cbStruct size */
			(UINT)5000, /* timeout */
			(UINT)DSP_SHMSEG0, /* segid */
			(UINT)0, /* allignment */
			(UINT)1, /* numbufs */

        (DSP_STRMMODE) STRMMODE_PROCCOPY, /* strm mode */

			(UINT)0, /* uDMAChnlId */
			(UINT)1, /* uDMAPriority - 1 -  */
		}   /* valid pointer using ZERO-COPY */
    };

    DSP_HSTREAM arg5[] =   /* arg 5: Location of returned stream handle */
    {
        NULL, /* valid location to be allocated */
        NULL                /* NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))
    #define NUMARG4 (sizeof(arg4)/sizeof(struct DSP_STREAMATTRIN))
    #define NUMARG5 (sizeof(arg5)/sizeof(DSP_HSTREAM))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] [NUMARG5] =
                                                /* arg1 arg2 arg3 arg4 arg5 */
    {
        {
            {
                {
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0][1] */
                    },
                    {
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1][0] */
                        {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][1][0][0] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][1][0][1] */
                    },
                    {
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][1][1][0] */
                        {DSP_EVALUE, TST_SHOULDEQUAL}, /* [0][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][0][0][0] */
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][0][0][1] */
                    },
                    {
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][0][1][0] */
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][1][0][0] */
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][1][0][1] */
                    },
                    {
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][1][1][0] */
                        {DSP_EDIRECTION, TST_SHOULDEQUAL}, /* [0][1][1][1][1] */
                    },
                },
            },
        },
        {
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}    /* [1][0][1][1][1] */
                    },
                },
            },
            {
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1][1] */
                    },
                },
                {
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0][1] */
                    },
                    {
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][0] */
                        {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1][1] */
                    },
                },
            },
        },
    };

    /* Allocate pointer to dummy attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));

    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    nodeAttrin.uTimeout = 10000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


    cbData = (struct DSP_CBDATA *)argsBuf;
    cbData->cbData = 32;

    /* NODE strm attrs: specify attrs for proccopy for now */
    attrs.uBufsize = 64;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uSegid = 0;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPStream_Open: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPStream_Open:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPStream_Open: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()\n");
             }
            break;
        }

        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
						NULL);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start()\n");
            }
            break;
        }

        pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Allocate().\n");
            }
            break;
        }

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, arg1[0], 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(arg1[0], 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        /* Create Node on DSP */
        status = DSPNode_Create(arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Create()\n");
            }
            break;
        }

        status = DSPNode_Run(arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Run()\n");
            }
            break;
        }

        /* Note: the following nested loops are rearranged such that
       the fourth parameter is run as the outer most loop.  The
       reason for doing this is because it requires that the DSP
       node be stopped, reconnected, and started again, and placing
       this functionality in the outer most loop minimizes the number
       of times the DSPNode has to be reconfigured
        */

        for (l = 0; l < NUMARG4; l++) {
            if (l == 1) {
                /* Start from scratch - reattach board, reallocate node */
                status = DSPNode_Terminate(arg1[0], &exitStatus);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Terminate FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPNode_Delete(arg1[0]);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Delete FAIL and status 0x%x\n",status);
			break;
		}

                status = DSPProcessor_Detach(hProcessor);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPProcessor_Detach FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPProcessor_Attach FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
								NULL);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPProcessor_Load FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPProcessor_Start(hProcessor);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPProcessor_Start FAIL and status 0x%x\n",status);
			break;
		}

                /* Specify attrs for zerocopy */
                attrs.uBufsize = 64;
                attrs.uNumBufs = 1;
                attrs.uAlignment = 0;
                attrs.uTimeout = DSP_FOREVER;
               
#if defined(OMAP_2430) || defined(OMAP_3430) 
                attrs.lMode = STRMMODE_PROCCOPY;
                attrs.uSegid = 0;
#else
                attrs.lMode = STRMMODE_ZEROCOPY;
                attrs.uSegid = DSP_SHMSEG0;
#endif
                status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData,
                                 &nodeAttrin, &arg1[0]);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Allocate FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, arg1[0],
                 0, &attrs);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Connect1 FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPNode_Connect(arg1[0], 0, (DSP_HNODE)DSP_HGPPNODE,
                                 0, &attrs);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Connect2 FAIL and status 0x%x\n",status);
			break;
		}

                status = DSPNode_Create(arg1[0]);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Create FAIL and status 0x%x\n",status);
			break;
		}
                status = DSPNode_Run(arg1[0]);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Run FAIL and status 0x%x\n",status);
			break;
		}
            }

            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (i = 0; i < NUMARG1; i++) {
                        for (m = 0; m < NUMARG5; m++) {

                            status = DSPStream_Open(arg1[i], arg2[j],
                                         arg3[k], &arg4[l], &arg5[m]);

                            if (!TST_CheckVal(status,
                                checkArray[i][j][k][l][m].dwValue,
                                checkArray[i][j][k][l][m].fEqualFlag)) {
                                cRetVal = 0;
                                TST_verbose(
                                "Open [%d][%d][%d][%d][%d]  %x\n",
                                        i, j, k, l, m, status);

                                if (bUseLog) {
                                    TSTLOG_appendLog(pHandle, "FAIL\n");
                                    TSTLOG_appendLog(pHandle,
                                    "DSPStream_Open [%d][%d][%d][%d][%d] \
                                    failed, returned %x\n", i, j, k, l, m, status);
                                }
                            }

                            if (DSP_SUCCEEDED(status)) {
                                /* Place in appropriate state in order
                                   to terminate */
                                status = DSPStream_Close(arg5[m]);
                            }
                        }
                    }
                }
            }
        }

        break;
    }

    		status = DSPNode_Terminate(arg1[0], &exitStatus);
		if(!DSP_SUCCEEDED(status))
		{
                	TSTLOG_appendLog(pHandle, "DSPNode_Terminate FAIL and status 0x%x\n",status);
		}
		else
		{
    			status = DSPNode_Delete(arg1[0]);
			if(!DSP_SUCCEEDED(status))
			{
                	TSTLOG_appendLog(pHandle, "DSPNode_Delete FAIL and status 0x%x\n",status);
			}
			else
			{
    				status = DSPProcessor_Detach(hProcessor);
				if(!DSP_SUCCEEDED(status))
				{
                		TSTLOG_appendLog(pHandle, "DSPProcessor_Detach FAIL and status 0x%x\n",status);
				}
		        }
		}
    
    OsSpecificMemFree(pAttrIn);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

