/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPStream_Select.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPStream\DSPStream_Select.c
 *
 *   Desc:
 *        DSPStream_Select API test
 *
 *   DSPStream API function tested:
 *        DSPStream_Select(DSP_HSTREAM *aStreamTab, UINT nStreams,
 *                         UINT *pMask, UINT uTimeout);
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ===========================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ===========================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFBUFSIZE     64
#define DEFBUFNUMS      1
#define DEFAULTDSPUNIT  0
#define DSPEXEC        "inout_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPStream_Select_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT                 opt;
    INT                 cRetVal = 1;
    INT                 i, j, k, l;
    INT                 iArgc = 1;
    BOOL                bUseLog=FALSE;
    FILE                *pHandle=NULL;
    CHAR                szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                *pszUuid;
    DSP_HPROCESSOR      hProcessor;
    DSP_HNODE           hNode = NULL;
    DSP_STATUS          status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_UUID            gidNodeID;
    struct DSP_CBDATA          *cbData;
    struct DSP_NODEATTRIN      nodeAttrin;
    struct DSP_STRMATTR        attrs;
    BYTE                argsBuf[32 + sizeof(ULONG)];


    /* INPUT PARAMETERS */
    DSP_HSTREAM *arg1[] =     /* arg 1: array of handles of a stream */
    {
        NULL, /* valid array to be allocated */
		NULL  /* invalid NULL array case not tested due to memory
                       access constraint. */
    };

    UINT arg2[] =   /* arg 2: number of stream handles in array */
    {
        2, /* valid number */
        2           /* invalid # not used as it causes array overflow */
    };

    UINT *arg3[] =  /* arg 3: pointer to location to receive mask */
    {
        NULL, /* valid pointer to be allocated */
        NULL        /* invalid pointer */
    };

    UINT arg4[] =   /* arg 4: timeout value */
    {
        0, /* valid number */
        0            /* invalid number */
    };

    BYTE **arg5[] =     /* arg 2: location for pointer to buffer */
    {
        NULL, /* valid buffer */
        NULL  /* valid buffer */
    };

    BYTE            ptrBuf[DEFBUFSIZE];
	BYTE            ptrBuf1[DEFBUFSIZE];
    ULONG           iNumBytes;
    DWORD           dwArg;


#define NUMARG1 (sizeof(arg1)/sizeof(DSP_HSTREAM *))
#define NUMARG2 (sizeof(arg2)/sizeof(UINT))
#define NUMARG3 (sizeof(arg3)/sizeof(UINT *))
#define NUMARG4 (sizeof(arg4)/sizeof(UINT))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0] */
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][0][0] */
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][1][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1] */
                    },
                },
        }
    };

    /* Allocate pointer to dummy attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));

    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


    cbData = (struct DSP_CBDATA *)argsBuf;
    cbData->cbData = 32;

    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;

    /* Allocate pointer location for stream mask */
    arg3[0] = OsSpecificMemAlloc(sizeof(UINT));

    arg1[0] = OsSpecificMemAlloc(sizeof(arg2[0]*sizeof(DSP_HSTREAM)));

    arg5[0] = OsSpecificMemAlloc(sizeof(BYTE *));
	arg5[1] = OsSpecificMemAlloc(sizeof(BYTE *));

    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
            case 'v':
                /* verbose */
                TST_setVerbose(TRUE);
                TST_verbose("DSPStream_Select: Verbose Mode On\n");
                break;
            case 'l':
                /* use log and store with name location */
                strncpy(szLogName, optarg, sizeof(szLogName));
                bUseLog = TRUE;
                TST_verbose("Opening %s\n", szLogName);
                pHandle = TSTLOG_openLog(szLogName);
                TSTLOG_appendLog(pHandle, "DSPStream_Select:\n");
                /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
                TST_verbose("Log opened as: %s\n", szLogName);
                break;
            case 'x':
                /* use given executable */
                aArgv[0] = (char *) optarg;
                break;
            default:
                TST_reportError(("Invalid option: %c\n"), opt);
                cRetVal = 0;

                if (bUseLog) {
                    TSTLOG_appendLog(pHandle,
                            "Invalid option specified.\n");
                }
                break;
        }
    }

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPStream_Select: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Attach()\n");
            }
            break;
        }

        arg1[1] = arg1[0];
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv,
						NULL);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Start()\n");
            }
            break;
        }

		pszUuid = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1";
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData,
            &nodeAttrin, &hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Allocate().\n");
            }
            break;
        }

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
            &attrs);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
            &attrs);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Connect()\n");
            }
            break;
        }


        /* Create Node on DSP */
        status = DSPNode_Create(hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Create()\n");
            }
            break;
        }

        status = DSPNode_Run(hNode);

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPNode_Run()\n");
            }
            break;
        }

        status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL,
            &((arg1[0])[0]));

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPStream_Open()\n");
            }
            break;
        }

        status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL,
            &((arg1[0])[1]));

        if (DSP_FAILED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPStream_Open()\n");
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {

                        status = DSPStream_Select(arg1[i], arg2[j],
                            arg3[k], arg4[l]);

                        if (!TST_CheckVal(status,
                            checkArray[i][j][k][l].dwValue,
                            checkArray[i][j][k][l].fEqualFlag)) {
                            cRetVal = 0;
                            TST_verbose(
                                "Select [%d][%d][%d][%d] returned %x\n",
                                i, j, k, l, status);

                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                "DSPStream_Select failed [%d][%d][%d][%d] \
                                failed.\n", i, j, k, l);
                            }
                        }
                    }
                }
            }
        }

        break;
    }


	/* Now, check the behaviour of the Select call - This is a new addition */
	/* Issue a Buffer on the Output stream(and keep it blocked).
	 * Call Select and it shoud select the input Stream.
	 * Now, Issue empty buffer on Input Stream and block that also.
	 * Call Select and it should not select any Stream
	 */

	 /* Alloc Buffer */
    status = DSPStream_AllocateBuffers(arg1[0][0], DEFBUFSIZE, arg5[0],
            DEFBUFNUMS);
    if (DSP_FAILED(status)) {
        /* Report Error */
        cRetVal = 0;
		TST_verbose("FAIL\n");

        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPStream_AllocateBuffers()\n");
		}
	}

    status = DSPStream_AllocateBuffers(arg1[0][1], DEFBUFSIZE, arg5[1],
            DEFBUFNUMS);
    if (DSP_FAILED(status)) {
        /* Report Error */
        cRetVal = 0;
		TST_verbose("FAIL\n");

        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPStream_AllocateBuffers()\n");
	    }
	}

    iNumBytes = DEFBUFSIZE;
    dwArg = 0x0;
	/* Fill Buffer */
    for (i = 0; i < (INT) iNumBytes; i++) {
	    ptrBuf[i] = (BYTE) 'a';
    }

	status = DSPStream_Select(arg1[0], arg2[0], arg3[0], arg4[0]);

	/* Issue Data on OUTPUT Stream */
	status = DSPStream_Issue(arg1[0][0], ptrBuf, iNumBytes, iNumBytes, dwArg);

    status = DSPStream_Select(arg1[0], arg2[0], arg3[0], arg4[0]);

    /* Reclaim output stream and issue back, to keep synched with 'inout' */
	status = DSPStream_Reclaim(arg1[0][0], (BYTE **)&ptrBuf, &iNumBytes,
            NULL, &dwArg);

	status = DSPStream_Issue(arg1[0][0], ptrBuf, iNumBytes, iNumBytes, dwArg);

	status = DSPStream_Issue(arg1[0][1], ptrBuf1, iNumBytes, iNumBytes, dwArg);

    status = DSPStream_Select(arg1[0], arg2[0], arg3[0], arg4[0]);

	OsSpecificDelay(1);

    status = DSPStream_Select(arg1[0], arg2[0], arg3[0], arg4[0]);

	/* Do the final reclaims now */
	status = DSPStream_Reclaim(arg1[0][1], (BYTE **)&ptrBuf1, &iNumBytes,
            NULL, &dwArg);
	status = DSPStream_Reclaim(arg1[0][0], (BYTE **)&ptrBuf, &iNumBytes,
            NULL, &dwArg);

	/* Free the Buffers used for Issue and Reclaim */
	status = DSPStream_FreeBuffers(arg1[0][0], arg5[0], DEFBUFNUMS);
	status = DSPStream_FreeBuffers(arg1[0][1], arg5[1], DEFBUFNUMS);

    status = DSPStream_Close(arg1[0][0]);
    status = DSPStream_Close(arg1[0][1]);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);
    status = DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(arg3[0]);
	OsSpecificMemFree(arg1[0]);
	OsSpecificMemFree(arg5[0]);
	OsSpecificMemFree(arg5[1]);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}
