/*  ============================================================================
 *   File    IVADSPManager_Close.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPManager\
 * 
 *   Desc:
 *           DSPManager API test
 *
 *           DSPManager API function tested:
 *           DSPManager_Close(UINT argc, PVOID argp)
 *
 *   Usage:
 *           DSPManager_Close [-v] [-l "log path/filename"]
 *
 *           -v: verbose mode
 *           -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   NOV 14, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>


#define DEFNAMESIZE    80

/*
 *  ======== DSPManager_Close_API ========
 *
 */
INT IVADSPManager_Close_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT         opt;
    INT         cRetVal = 1;
    INT         i, j;
    BOOL        bUseLog=FALSE;
    FILE        * pHandle;
    CHAR        szLogName[DEFNAMESIZE];
    DSP_STATUS  status;

    /* INPUT PARAMETERS */

    /* INVALID ranges no longer detected since ranges can vary */
    UINT arg1[] =               /* arg1: numeric node ID range */
    {
        0,                      /* Valid number */
        50,                     /* valid number */
    };

    PVOID *arg2[] =      /* arg2: pointer to node info storage */
    {
        NULL,                   /* valid pointer to be allocated */
        NULL                    /* invalid NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(UINT))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))   /* UINT - taken arbitrarily */

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] =

    {
        {
            {DSP_SOK, TST_SHOULDEQUAL},    /* [0][0] */
            {DSP_SOK, TST_SHOULDEQUAL},    /* [0][1] */
        },
        {
            {DSP_SOK, TST_SHOULDEQUAL},    /* [1][0] */
            {DSP_SOK, TST_SHOULDEQUAL},    /* [1][1] */
        }
    };


    /* Initialise pHandle */
    pHandle = (FILE *)NULL;

    /* Allocate pointer to arbitrary structure */
    arg2[0] = OsSpecificMemAlloc(sizeof(UINT));

    /* Reset getopt pointer */
    optind = 0;

    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:")) != EOF) 
	{

        switch (opt) 
		{
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("IVADSPManager_Close: Verbose Mode On\n");
            break;
            
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "IVADSPManager_Close:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
            
        default:
            TST_verbose("Invalid option: %c\n", opt);
            cRetVal = 0;
            if (bUseLog) 
			{
                TSTLOG_appendLog(pHandle, "\nInvalid option specified.\n");
            }
            break;
        }
    }

    TST_verbose("IVADSPManager_Close: ");
    TST_verbose("%i: ", iRunTime);

    if (cRetVal > 0) 
	{
        for (i = 0; i < NUMARG1; i++) 
        {
            for (j = 0; j < NUMARG2; j++) 
            {
				status = DspManager_Open(0, 0);

                status = DspManager_Close(arg1[i], arg2[j]);

                if (!TST_CheckVal(status,
                    checkArray[i][j].dwValue,
                    checkArray[i][j].fEqualFlag)) 
                {
                    cRetVal = 0;
                    TST_verbose("Close [%d] [%d] "
                        "returned %x\n", i, j, status);
                    if (bUseLog) 
                    {
                        TSTLOG_appendLog(pHandle, "FAIL\n");
                        TSTLOG_appendLog(pHandle,
                            "IVADSPManager_Close [%d][%d] "
                            "failed.\n", i, j);
                    }
                }
            }
        }
    }

    OsSpecificMemFree(arg2[0]);

    if (cRetVal != 0) {

        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

