/*  ============================================================================
 *   File    IVADSPManager_EnumNodeInfo.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPManager\
 * 
 *   Desc:
 *           DSPManager API test
 *
 *           DSPManager API function tested:
 *           DSPManager_EnumNodeInfo(UINT uNode, DSP_NDBPROPS *pNDBProps,
 *                              UINT uNDBPropsSize, UINT *puNumNodes)
 *
 *   Usage:
 *           DSPManager_EnumNodeInfo [-v] [-l "log path/filename"]
 *
 *           -v: verbose mode
 *           -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>


#define DEFNAMESIZE    80

/*
 *  ======== main ========
 *
 */
INT IVADSPManager_EnumNodeInfo_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT         opt;
    INT         cRetVal = 1;
    INT         i, j, k, l;
    BOOL        bUseLog=FALSE;
    FILE        * pHandle;
    CHAR        szLogName[DEFNAMESIZE];
    DSP_STATUS  status;

    /* INPUT PARAMETERS */

    /* INVALID ranges no longer detected since ranges can vary */
    UINT arg1[] =               /* arg1: numeric node ID range */
    {
	1,                      /* Valid number */
        50,                     /* valid number */
    };

    struct DSP_NDBPROPS *arg2[] =      /* arg2: pointer to node info storage */
    {
        NULL,                   /* valid pointer to be allocated */
        NULL                    /* invalid NULL pointer */
    };

    UINT arg3[] =               /* arg3: NDBPROPS size */
    {
        sizeof(struct DSP_NDBPROPS),   /* valid size */
        0                       /* valid size */
    };

    UINT *arg4[] =              /* arg4: pointer to # nodes in database */
    {
        NULL,                   /* Valid pointer to be allocated */
        NULL                    /* Invalid NULL pointer */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(UINT))
    #define NUMARG2 (sizeof(arg2)/sizeof(struct DSP_NDBPROPS *))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))
    #define NUMARG4 (sizeof(arg4)/sizeof(UINT *))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},      /* [0][0][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][0][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},    /* [0][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][1][1][1] */
                },
            },
        },
        {
            {
                {
                    {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [1][0][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][0][0][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},    /* [1][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][0][1][1] */
                },
            },
            {
               {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][1][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][1][0][1] */
               },
               {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][1][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [1][1][1][1] */
               },
            },
        },
    };


    /* Initialise pHandle */
    pHandle = (FILE *)NULL;

    /* Allocate pointer to node information storage structure */
	arg2[0] = OsSpecificMemAlloc(sizeof(struct DSP_NDBPROPS));

    /* Allocate pointe for node number storage */
	arg4[0] = OsSpecificMemAlloc(sizeof(UINT *));
	
    /* Reset getopt pointer */
    optind = 0;

    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:")) != EOF) {
        
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("IVADSPManager_EnumNodeInfo: Verbose Mode On\n");
            break;
            
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "IVADSPManager_EnumNodeInfo:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
            
        default:
            TST_verbose("Invalid option: %c\n", opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "\nInvalid option specified.\n");
            }
            break;
        }
    }

    TST_verbose("IVADSPManager_EnumNodeInfo: ");
    TST_verbose("%i: ", iRunTime);

    if (cRetVal > 0) {
        
        for (i = 0; i < NUMARG1; i++) {
            
            for (j = 0; j < NUMARG2; j++) {
                
                for (k = 0; k < NUMARG3; k++) {
                    
                    for (l = 0; l < NUMARG4; l++) {
                        status = DSPManager_EnumNodeInfo(arg1[i], arg2[j],
                            arg3[k], arg4[l]);
                            
	/*		TSTLOG_appendLog(pHandle,"In Main loop Iteration [%d] [%d] [%d] [%d] and returned status is  %x\n", i, j, k, l, status);*/

                        if (!TST_CheckVal(status,
                            checkArray[i][j][k][l].dwValue,
                            checkArray[i][j][k][l].fEqualFlag)) {
                            cRetVal = 0;
                            TST_verbose("EnumNodeInfo [%d] [%d] [%d] [%d] " "returned %x\n", i, j, k, l, status);
                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                    "IVADSPManager_EnumNodeInfo [%d][%d][%d][%d] "
                                    "failed.\n", i, j, k, l);
                            }
                        }
                    }
                }
            }
        }
    }

	OsSpecificMemFree(arg2[0]);
	OsSpecificMemFree(arg4[0]);
    if (cRetVal != 0) {

        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

