/*  ============================================================================
 *   File    IVADSPProcessor_Load.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPProcessor
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_Load(DSP_HPROCESSOR hProcessor, INT iArgc,
 *                          CHAR **aArgv, CHAR **aEnvp);
 *
 *  Usage:
 *        s framework API [-v] [-l "log path/filename"] [-x "valid executable"]
 *        [-m "DSPProcessor"] [-n "7"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: name of valid DSP executable
 *        -m: Module Name - in this case "DSPProcessor"
 *        -n: Number of the API to be tested (7)
 *
 *         Please refer to apitst.c for further details
 *
 *  Issues:
 *        When arg2[1] is set to '0', which should be indicative of
 *        an invalid argument for DSPProcessor_Load, instead of failing
 *        normally, the module throws an exception.  This has only
 *        occured when this test is run as a DLL and not as an EXE.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>


#define DEFNAMESIZE    80

#define DSPEXEC		"iva.dof"

/*
 *  ======== main ========
 *
 */
INT IVADSPProcessor_Load_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i, j, k, l;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN *pAttrIn;

    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;

    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =     /* arg1: handle of attached processor */
    {
        NULL, /* Valid handle to be attached */
        NULL, /* Invalid NULL handle */
    };

    /* INVALID '0' for number of arguments reports exception */
    INT arg2[] =                /* arg2: number of arguments */
    {
        1, /* valid number of arguments */
        0                       /* invalid number of arguments */
    };

    CHAR **arg3[] =             /* arg3: array of arguments */
    {
        aArgv, /* Valid array */
        NULL                    /* invalid NULL array */
    };

    /* INVALID array checking not done since NULL = default settings */
    CHAR **arg4[] =             /* arg4: array of env settings */
    {
        NULL, /* valid array */
        NULL  /* valid NULL array */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(INT))
    #define NUMARG3 (sizeof(arg3)/sizeof(CHAR **))
    #define NUMARG4 (sizeof(arg4)/sizeof(CHAR **))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][0] */
                    {DSP_SOK, TST_SHOULDEQUAL}, /* [0][0][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL}, /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [0][1][0][0] */
                    {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [0][1][0][1] */
                },
                {
                    {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [0][1][1][0] */
                    {DSP_EINVALIDARG, TST_SHOULDEQUAL}, /* [0][1][1][1] */
                },
            },
        },
        {
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1] */
                },
            },
            {
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1] */
               },
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1] */
               },
            },
        },
    };

    /* Allocate pointer to dummy set of attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
         switch (opt) {
         case 'v':
             /* verbose */
             TST_setVerbose(TRUE);
             TST_verbose("DSPProcessor_Load: Verbose Mode On\n");
             break;
      case 'l':
             /* use log and store with name location */
             strncpy(szLogName, optarg, sizeof(szLogName));
          bUseLog = TRUE;
             TST_verbose("Opening %s\n", szLogName);
             pHandle = TSTLOG_openLog(szLogName);
             TSTLOG_appendLog(pHandle, "DSPProcessor_Load:\n");
             /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
             TST_verbose("Log opened as: %s\n", szLogName);
          break;
         case 'x':
             /* use given DSP executable */
         (arg3[0])[0] = (char *) optarg;
             break;
         default:
             TST_verbose(("Invalid option: %c\n"), opt);
             cRetVal = 0;
             if (bUseLog) {
                 TSTLOG_appendLog(pHandle,
                 "Invalid Option Specified.");
             }
             break;
        }
    }

    TST_verbose("DSP Executable: %s\n", (arg3[0])[0]);
    TST_verbose("DSPProcessor_Load: ");
    TST_verbose("%i: ", iRunTime);

    if (cRetVal > 0) {

/////////////////////////////////////////////////////////////////////
	 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
          {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
          }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId,pAttrIn,&arg1[0]);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
	    cRetVal=0;
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");


/////////////////////////////////////////////////////////////////////

        if(DSP_SUCCEEDED(status)) {
            for (i = 0; i < NUMARG1; i++) {
                for (j = 0; j < NUMARG2; j++) {
                    for (k = 0; k < NUMARG3; k++) {
                        for (l = 0; l < NUMARG4; l++) {
                            status = DSPProcessor_Load(arg1[i], arg2[j],
                                     (CONST CHAR **)arg3[k], (CONST CHAR **)arg4[l]);

                            if (!TST_CheckVal(status,
                               checkArray[i][j][k][l].dwValue,
                               checkArray[i][j][k][l].fEqualFlag)) {
                               cRetVal = 0;
                               TST_verbose(
                               "Load [%d][%d][%d][%d] returned %x\n",
                                              i, j, k, l, status);

                               if (bUseLog) {
                                   TSTLOG_appendLog(pHandle, "FAIL\n");
                                   TSTLOG_appendLog(pHandle,
                                   "DSPProcessor_Load \
                                   [%d][%d][%d][%d] failed.\n", i, j, k, l);
                               }
                            }
                        }
                    }
                }
            }

            /* Detach Processor */
            status = DSPProcessor_Detach(arg1[0]);
        }
    }

    OsSpecificMemFree(pAttrIn);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

