/*  ============================================================================
 *   File    IVADSPProcessor_RegisterNotify.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPProcessor\
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *        DSPProcessor_RegisterNotify(DSP_HPROCESSOR hProcessor,
 *                                    UINT uEventMask,
 *                                    UINT uNotifyMask,
 *                                    DSP_HNOTIFICATION hNotification)
 *  Usage:
 *        DSPProcessor_RegisterNotify [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbtype.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>

#define DEFNAMESIZE    80


INT ivatmpDSPProcessor_RegisterNotify_API (INT argc, CHAR * argv[], INT iRunTime)
{
	return 0;
}

/*
 *  ======== main ========
 *
 */
INT IVADSPProcessor_RegisterNotify_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT                     opt;
    INT                     cRetVal = 1;
    INT                     i, j, k, l;
    BOOL                    bUseLog = FALSE;
    FILE                    * pHandle=NULL;
    CHAR                    szLogName[DEFNAMESIZE];
    DSP_STATUS              status;
    struct DSP_PROCESSORATTRIN     * pAttrIn;
    struct DSP_NOTIFICATION        hNotification;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =     /* arg1: handle of attached processor */
    {
        NULL,                   /* Valid handle to be attached */
        NULL,                   /* Invalid NULL handle */
    };

    UINT arg2[] =               /* arg2: uEventMask */
    {
        DSP_PROCESSORSTATECHANGE |
        DSP_PROCESSORATTACH |
        DSP_PROCESSORDETACH,     /* valid mask */
        0xffff                  /* invalid mask */ /* Changed from Zero to ffff */
    };

    UINT arg3[] =               /* arg3: uNotifyMask */
    {
        2 |						/*DSP_POSTMESSAGE  is changed to 2*/
        DSP_SIGNALEVENT,        /* Valid mask */
        0                       /* Invalid NULL mask */
    };

    struct DSP_NOTIFICATION* arg4[] =  /* arg4: Notification handle */
    {
        NULL,                   /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))
    #define NUMARG3 (sizeof(arg3)/sizeof(UINT))
    #define NUMARG4 (sizeof(arg4)/sizeof(struct DSP_NOTIFICATION*))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},     /* [0][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][0][1] */
                },
                {
                    {DSP_ENOTIMPL, TST_SHOULDEQUAL},/* [0][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL},   /* [0][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][0][1] */
                },
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL},/* [0][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [0][1][1][1] */
                },
            },
        },
        {
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1] */
                },
            },
            {
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][0][1] */
               },
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][1][1][1] */
               },
            },
        },
    };

    /* Allocate pointer to dummy set of attributes */

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
         switch (opt) {
         case 'v':
             /* verbose */
             TST_setVerbose(TRUE);
             TST_verbose("DSPProcessor_RegisterNotify: Verbose Mode On\n");
             break;
      case 'l':
             /* use log and store with name location */
             strncpy(szLogName, optarg, sizeof(szLogName));
              bUseLog = TRUE;
             TST_verbose("Opening %s\n", szLogName);
             pHandle = TSTLOG_openLog(szLogName);
             TSTLOG_appendLog(pHandle, "DSPProcessor_RegisterNotify:\n");
             /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
             TST_verbose("Log opened as: %s\n", szLogName);
          break;
         case 'x':
             /* Use DSP executable */

             /*
              *  Not used in this test - SKIP
              *  Leave option here, however, in case executable argument
              *  is passed in from 'apitst'
              */

             break;
         default:
             TST_verbose(("Invalid option: %c\n"), opt);
             cRetVal = 0;
             if (bUseLog) {
                 TSTLOG_appendLog(pHandle,
                 "Invalid Option Specified.\n");
             }
             break;
         }
    }

    TST_verbose("DSPProcessor_RegisterNotify: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {

//////////////////////////////////
	 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
           {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
           }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, pAttrIn,&arg1[0]);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
	    cRetVal=0;
	    break;
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

//////////////////////////////////
        /* Allocate notification handle */
        arg4[0] = &hNotification;
        /* Since DSP_POSTMESSAGE is not yet supported, hard code the valid
		 * Event mask as DSP_SIGNALEVENT
		 */
        arg3[0] = DSP_SIGNALEVENT;

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {
                for (k = 0; k < NUMARG3; k++) {
                    for (l = 0; l < NUMARG4; l++) {

                        status = DSPProcessor_RegisterNotify(arg1[i],
                                 arg2[j], arg3[k], arg4[l]);

                        if (!TST_CheckVal(status,
                            checkArray[i][j][k][l].dwValue,
                            checkArray[i][j][k][l].fEqualFlag)) {
                            cRetVal = 0;
                            TST_verbose(
                                "RegisterNotify [%d][%d][%d][%d] "
                                "returned %x\n", i, j, k, l, status);

                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                "DSPProcessor_RegisterNotify [%d][%d][%d][%d] \
                                failed.\n", i, j, k, l);
                            }
                        }
                    }
                }
            }
        }
        /* Detach Processor */
        status = DSPProcessor_Detach(arg1[0]);
        break;
    }
    
	OsSpecificMemFree(pAttrIn);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

