/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    tstlog.c
 *
 *   Path    $(TESTSUITEROOT)\Testutil\tstlog\tstlog.c
 * 
 *   Desc:
 *        Test LOG utility for recording information to input log file.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2002
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#ifndef __KERNEL__
#include <std.h>
#include <stdio.h>
#include <dbtype.h>
#include <stdarg.h>
#include <tstlog.h>
#else
#include <stdio.h>
#include <dbtype.h>
#include <stdarg.h>
#include <linux/kernel.h>
#endif

/*
 *  ======= TSTLOG_openLog ========
 *  Purpose:
 *      Open logfile as name provided by string parameter
 *
 */
#ifndef __KERNEL__
FILE* TSTLOG_openLog(CHAR *szLogName)
{
    /* Open LOG File and return descriptor */
    FILE *pLog;
    
    /* Check to see if file exists.  If not, create it.  Else, open it */
    if ((pLog = fopen(szLogName, "a")) == NULL) {
        printf("Error Opening Output File: %s\n", szLogName);
        exit(2);
    }
    
    return pLog;
}
#else
VOID* TSTLOG_openLog(CHAR *szLogName)
{
	return NULL;
}
#endif


/*
 *  ======== TSTLOG_appendLog ========
 *  Purpose:
 *      Append data to opened logfile
 *
 */
#ifndef __KERNEL__
VOID TSTLOG_appendLog(FILE *pLog, CHAR *szData, ...)
#else
VOID TSTLOG_appendLog(VOID *pLog, CHAR *szData, ...)
#endif
{
    /* Write data to LOG file with loghandle */
    va_list va;
    CHAR tempBuf[1024];
    
    va_start(va, szData);
    vsprintf(tempBuf, szData, va);
#ifndef __KERNEL__
    // fprintf(pLog, tempBuf); - needed for future use
	printf(tempBuf);
#else
	printk(tempBuf);
#endif
    
    va_end(va);
}


/*
 *  ======== TSTLOG_closeLog ========
 *  Purpose:
 *      Close logfile with handle given as parameter
 *
 */
#ifndef __KERNEL__
VOID TSTLOG_closeLog(FILE *pLog)
{
    /* Close loghandle */
    fclose(pLog);   
}
#else
VOID TSTLOG_closeLog(VOID *pLog)
{
}
#endif

