#!/bin/bash


export KERNELDIR=/vobs/wtbu/CSSD_L_GIT_2.6/linux/kernel_org/2.6_kernel/
export OMAPDEV=/data/omapts/linux/arm-tc
export CSCCVER=arm-2008q3-72
export OMAPCSCC=$OMAPDEV/$CSCCVER/bin/
export CROSS_COMPILE=arm-none-linux-gnueabi-
export CT_DIR=/usr/atria/bin/
export BRIDGEDIR=/vobs/wtbu/OMAPSW_MPU/dspbridge
export PATH=$PATH:$HOME/bin:$OMAPDEV:$CT_DIR:$OMAPCSCC


export OMAPDIR="/vobs/wtbu"
export OMAP_L="$OMAPDIR/OMAPSW_L"
export OMAP_L2="$OMAPDIR/CSSD_L_GIT_2.6"
export OMAP_MPU="$OMAPDIR/OMAPSW_MPU"
export KRNLSRC=$KERNELDIR

clear;
echo "#############################"; 
echo "ez_build: Exported variables:"
echo "#############################"; 
echo "KERNELDIR: $KERNELDIR"
echo "OMAPDEV: $OMAPDEV"
echo "CSCCVER: $CSCCVER"
echo "OMAPCSCC: $OMAPCSCC"
echo "CROSS_COMPILE: $CROSS_COMPILE"
echo "CT_DIR: $CT_DIR"
echo "BRIDGEDIR: $BRIDGEDIR"
echo "PATH: $PATH"
echo "OMAPDIR: $OMAPDIR"
echo "OMAP_L: $OMAP_L"
echo "OMAP_L2: $OMAP_L2"
echo "OMAP_MPU: $OMAP_MPU"
sleep 3;

DO_UBOOT=0;
DO_KERNEL=0;
DO_BDRIVER=0;
DO_SAMPLES=0;
PM_OFF=0;
DO_KERN20=0;
DO_TESTSUITES=0;
DO_DSPSTUFF=0;
DO_COMPRESS=0;
HELP=0;
COPY=0;
BPATH=".";
DO_MAIL=0;
DO_BASEIMG=0;

if  [ "$#" = "0" ]; then
	clear;
	echo "#########################################"; 
	echo "# No parameters, building all for 3430. #";
	echo "#   ./ez_build -u -k -d -s -t -dsp -b   #";
	echo "#########################################";
	sleep 3;
	DO_UBOOT=1;
	DO_KERNEL=1;
	DO_BDRIVER=1;
	DO_SAMPLES=1;
	DO_TESTSUITES=1;
	DO_DSPSTUFF=1;
	DO_BASEIMG=1;
else 
	for I in $@; do
	 if [ "$COPY" = "2" ]; then 
	 	BPATH=$I;
		COPY=1;
		continue;
	 fi
	
	 if [ "$DO_MAIL" = "2" ]; then 
	 	ADDR=$I;
		DO_MAIL=1;
		continue;
	 fi
	
	 if [ "$PM_OFF" = "1" ]; then
#	 	if [ "$I" = "-* ]; then
	 	CMDS="$CMDS $I";
		continue;
	 fi

	 case $I in 
		-u) DO_UBOOT=1 ;;
		-k) DO_KERNEL=1 ;;
		-t) DO_TESTSUITES=1 ;;
		-s) DO_SAMPLES=1 ;;
		-d) DO_DRIVER=1 ;;
		-b) DO_BASEIMG=1 ;;
		-dsp)DO_DSPSTUFF=1 ;;
		-arg) PM_OFF=1 ;;
		-20) DO_KERN20=1;;
		-z) DO_COMPRESS=1 ;;
		-m) DO_MAIL=2;DO_COMPRESS=1 ;;
		-c) COPY=2 ;;
		-h) HELP=1 ;;
		*) { echo "Unknow option use -h for help." ; exit 1; } ;;
	 esac
	done
fi

if [ "$HELP" = 1 ]; then
	echo "Params:";
	echo "-u compile u-boot";
	echo "-k compile 3430 linux kernel";
	echo "-20 enable 3410 linux kernel, driver & baseimage";
	echo "-t compile bridge testsuites";
	echo "-s compile linux bridge samples";
	echo "-b compile MM baseimage";
	echo "-d compile linux bridge driver";
	echo "-arg <FLAGS> Lets the user to provide particular build flags like:";
	echo "		GT_TRACE DEBUG DISABLE_BRIDGE_PM DEBUG_BRIDGE_PERF";
	echo -e "\nIMPORTANT: Please use this option at the end of command line. !!\n";
	echo "-z compress(mail option  and parameters omission(make all) compress for default)";
	echo "-c <PATH> copy all to path(the PATH never have to end with '/')";
	echo "-m <ADDRESS> send a mail to ADDRESS, the message is in 'script/msg' file)";
	echo "-dsp compile dsp testcases and samples";
	echo "";
	echo "example: ./dailybuild.sh -u -k -d";
	echo "(for compiling u-boot, linux kernel and bridge driver)";
	exit 0;
fi

if [ "$COPY" = "0" ]; then
	BPATH="$BPATH/dspbridge_`date +%Y%m%d`";
fi

if test ! -d $BPATH; then
	mkdir $BPATH;
fi
  
if [ "$DO_UBOOT" = "1" ]; then 
	echo "###########################"; 
	echo "#     Compiling u-boot.   #";
	echo "###########################";
	if [ "$DO_KERN20" = "1" ]; then
		echo "###########################"; 
		echo "# Changed to 3410 u-boot. #";
		echo "###########################";
		#Actually defconfig is the same (3430) but I keep in case BE changes it
		./scripts/u_boot2.sh $OMAP_L || { echo "Error uboot script2 not found or failed."; exit 1; }
	else
		./scripts/u_boot.sh $OMAP_L || { echo "Error uboot script not found or failed."; exit 1; }
	fi
	cp $OMAP_L/u-boot/u-boot.bin $BPATH/


fi

if [ "$DO_KERNEL" = "1" ]; then
	echo "###########################"; 
	echo "# Compiling Linux kernel. #";
	echo "###########################";
	if test ! -f $OMAP_L/u-boot/tools/mkimage; then
		echo "mkimage not found in:"
		echo "$OMAP_L/u-boot/tools/"
		echo "please build u-boot before building kernel";
		exit 1;
	fi
	if test ! -d ~/bin; then
		mkdir ~/bin
	fi
	cp $OMAP_L/u-boot/tools/mkimage ~/bin
		
	if [ "$DO_KERN20" = "1" ]; then
		echo "###########################"; 
		echo "# Changed to 3410 kernel. #";
		echo "###########################";
		./scripts/linux_kernel2.sh $OMAP_L2 || { echo "Error linux kernel script2 not found or failed."; exit 1; }
	else
		./scripts/linux_kernel.sh $OMAP_L2 || { echo "Error linux kernel script not found or failed."; exit 1; }
	fi
	cp $OMAP_L2/linux/kernel_org/2.6_kernel/arch/arm/boot/uImage $BPATH/
fi

if [ "$DO_DRIVER" = "1" ]; then
	echo "###########################"; 
	echo "# Compiling Bridge Driver.#";
	echo "###########################";
	if test ! -d $BPATH/libs; then
		mkdir $BPATH/libs
	fi

	if [ "$PM_OFF" = "1" ]; then
		echo "##################################################"; 
		echo "#  New build arguments $CMDS. #";
		echo "##################################################";
		./scripts/bridge_driver2.sh $OMAP_MPU "$OMAP_L2/linux/kernel_org/2.6_kernel" "$CMDS" || { 
		echo "ErrorBridge driver script2 not found or failed."; 
		echo "*** Please make you built the kernel first***"; exit 1; }
	else
	./scripts/bridge_driver.sh $OMAP_MPU "$OMAP_L2/linux/kernel_org/2.6_kernel" || { 
		echo "Error Bridge driver script not found or failed.";
		echo "*** Please make sure you built the kernel first***"; exit 1; }
	fi
	cp $OMAP_MPU/dspbridge/mpu_driver/src/bridgedriver.ko $BPATH/
	cp -x $OMAP_MPU/dspbridge/target/lib/libbridge* $BPATH/libs/ 
	cp -x $OMAP_MPU/dspbridge/target/lib/libqos* $BPATH/libs/ 
	cp -x $OMAP_MPU/dspbridge/target/lib/libpthread* $BPATH/libs/
fi

if [ "$DO_TESTSUITES" = "1" ]; then 
	echo "###########################"; 
	echo "#  Compiling Testsuites.  #";
	echo "###########################";
	if test ! -d $BPATH; then
		mkdir $BPATH
	fi
	./scripts/bridge_testsuites.sh $OMAP_MPU "$OMAP_L2/linux/kernel_org/2.6_kernel" || { echo "Error testsuites script not found or failed."; exit 1; }
	cp $OMAP_MPU/dspbridge/testmpu/src/Framework/framework.out $BPATH/
	cp $OMAP_MPU/dspbridge/testmpu/src/script/* $BPATH/
fi

if [ "$DO_SAMPLES" = "1" ]; then
	echo "###########################"; 
	echo "#Compiling Bridge samples.#";
	echo "###########################";
	if test ! -d $BPATH; then
		mkdir $BPATH
	fi
	./scripts/bridge_samples.sh $OMAP_MPU "$OMAP_L2/linux/kernel_org/2.6_kernel" || { echo "Error samples script not found or failed."; exit 1; }
	cp $OMAP_MPU/dspbridge/target/dspbridge/*.out $BPATH/
fi

if [ "$DO_DSPSTUFF" = "1" ]; then
	echo "###########################"; 
	echo "# Compiling DSP code.     #";
	echo "###########################";
	if test ! -d $BPATH; then
		mkdir $BPATH
	fi
	./scripts/dsp_code.sh $OMAP_MPU "$OMAP_L2/linux/kernel_org/2.6_kernel" || { echo "Error dsp code script not found or failed."; exit 1; }
	cp -x /vobs/SDS/Source/Bridge/dsp/ti/dspbridge/dsp/samples/*64P $BPATH/
	cp -x /vobs/SDS/Source/Bridge/dsp/ti/dspbridge/dsp/qos/*64P $BPATH/
	cp -x /vobs/SDS/Source/Bridge/dsp/ti/dspbridge/dsp/tests/testsuite/*64P $BPATH/
fi


if [ "$DO_BASEIMG" = "1" ]; then
	echo "###########################"; 
	echo "# Compiling MM baseimage. #";
	echo "###########################";
	if test ! -d $BPATH/libs; then
		mkdir $BPATH/libs
	fi
	if test ! -f /vobs/SDS/Source/Bridge/dsp/ti/dspbridge/dsp/tests/testsuite/*64P; then
		echo "Error: Please make sure you already built DSP binaries"
		exit 1;
	fi
		
	if [ "$DO_KERN20" = "1" ]; then
		echo "###########################"; 
		echo "#   Changed to 3410 MMBI  #";
		echo "###########################";
		./scripts/mm_baseimage2.sh $OMAP_L2 || { echo "Error linux kernel script2 not found or failed."; exit 1; }
		cp -x /vobs/wtbu/OMAPSW_DSP/system/baseimage/out/omap3410/debug/baseimage.dof $BPATH/libs/ 
	else	
		./scripts/mm_baseimage.sh  || { echo "Error mm_baseimage script not found or failed."; exit 1; }
		cp -x /vobs/wtbu/OMAPSW_DSP/system/baseimage/out/omap3430/debug/baseimage.dof $BPATH/libs/ 
	fi
fi


echo "###########################"; 
echo "#     Saving ConfigSpec   #";
echo "###########################"; 
if test -f cs_.txt; then
	rm -f cs_.txt
fi
cleartool catcs 1> $BPATH/cs_.txt
if test -f scripts/msg; then
	rm -f msg
fi
echo "Daily build complete using next configs:" > scripts/msg.txt
cat $BPATH/cs_.txt >> scripts/msg.txt

if [ "$DO_COMPRESS" = "1" ]; then 
	rm -f "$BPATH.tor";
	tar -cf "$BPATH.tor" $BPATH
	#gzip "$BPATH.tar"
	rm -fr $BPATH
fi

if [ "$DO_MAIL" = "1" ]; then 
	mutt -a "$BPATH.tor" -s "DailyBuild(`date +%Y%m%d`) OK"  $ADDR < scripts/msg.txt
fi

exit 0;
